import { InputText } from 'primereact/inputtext';
import  {Component} from 'react';
import { Redirect } from 'react-router-dom';
import Auth from '../authenticate/auth';
import { CustomPageSpinner } from '../components/CustomPageSpinner';
import { InputSwitch } from 'primereact/inputswitch';
import PropTypes from 'prop-types';

/**
 * Component to authenticate users in the application.
 */
export class Login extends Component {
    KEYCLOAK_AUTH_URL= '/oidc/authenticate/';
    constructor(props){
        super(props);
        this.state = {
            username: "",
            password: "",
            redirect: null,          //If already logged in, redirect to home page
            error: null,
            errors: {},
            validFields: {},
            systemLogin: false
        };
        this.login = this.login.bind(this);
        this.setCredentials = this.setCredentials.bind(this);
        this.formSubmit = this.formSubmit.bind(this);
        this.doKeyCloakLogin = this.doKeyCloakLogin.bind(this);
    }

    /**
     * To set form field values.
     * @param {String} field 
     * @param {String} value 
     */
    setCredentials(field, value) {
        let state = this.state;
        let errors = state.errors;
        let validFields = state.validFields;
        // If value is null or empty set error field and remove from valid field and vice versa
        if (!value) {
            errors[field] = `${field} required`;
            delete validFields[field];
        }   else {
            delete errors[field];
            validFields[field] = field;
        }
        state[field] = value;
        state.errors = errors;
        state.validFields = validFields;
        this.setState(state);
    }

    /**
     * Function to call login function on Enter key press.
     * @param {React.KeyboardEvent} event 
     */
    formSubmit(event) {
        if (event.key === "Enter" && this.state.username && this.state.password) {
            this.login();
        }
    }

    /**
     * Login function called on click of 'Login' button. 
     * If authenticated, callback parent component function.
     */
    async login() {
        this.setState({showSpinner: true})
        const loggedIn = await Auth.login(this.state.username, this.state.password);
        if (loggedIn) {
            this.setState({error: false, showSpinner: false});
            this.props.onLogin();
        }   else {
            this.setState({error: true, showSpinner: false});
        }
    }

    /**
     * Keycloak login, redirect to keycloak page
     * @param {*}
     */
    async doKeyCloakLogin() {
        window.location.href = this.KEYCLOAK_AUTH_URL;
    }

    render() {
        if (this.state.redirect) {
            return (<Redirect to={{pathname: this.state.redirect}} />);
        }
        return (
            <>
            <div className="container-fluid bg-login">
                <div className="container">
                    <div className="row">
                        <div className="col-lg-9 col-md-12 login-card" style={{'height': '402px'}}>
                            <div className="row">
                                {/* Left panel with image and TMSS title */}
                                <div className="col-md-5 detail-part">
                                    <h3>Telescope Manager Specification System</h3>
                                    <p>By ASTRON</p>
                                </div>
                                {/* Right panel with login form */}
                                <div className="col-md-7 logn-part" style={{'height': '402px'}}>
                                    <div className="row login-part">
                                        <div className="col-lg-10 col-md-12 mx-auto">
                                            <div className="logo-cover">
                                                {/* <img src="./logo.png" alt="" /> */}
                                            </div>
                                            <div className="login-form">
                                                <h4>Login</h4>
                                                <div className="form-field" style={{height: '16em'}}>
                                                    <div className="row">
                                                        <label htmlFor="systemLogin" style={{marginLeft: '1em', marginRight: '1em'}}>Login with Keycloak</label>
                                                        <InputSwitch id="systemLogin" checked={this.state.systemLogin} onChange={(e) => { this.setState({'systemLogin' : e.value}) }}
                                                            tooltip={this.state.systemLogin ? 'Login as TMSS user':'Login with Keycloak'}
                                                            tooltipOptions={this.tooltipOptions}/>
                                                       <label htmlFor="systemLogin" style={{marginLeft: '1em', marginRight: '1em'}}>Login as TMSS user</label>
                                                    </div>
                                                    {!this.state.systemLogin &&
                                                        <div className="col-md-12 button-div" style={{textAlign: 'center', marginTop: '50px'}} >
                                                            <button className="btn btn-primary" id="KeyCloakLoginButton"
                                                                onClick={this.doKeyCloakLogin}>
                                                                    Go to Keycloak Login</button>
                                                        </div>
                                                    }
                                                    {this.state.systemLogin &&
                                                        <div>
                                                            <div className="form-field">
                                                                <span className="p-float-label">
                                                                    <InputText id="username" className={`${this.state.errors.username?"input-error ":""} form-control`} autoFocus
                                                                                value={this.state.username} onChange={(e) => this.setCredentials('username', e.target.value)}
                                                                                onKeyUp={this.formSubmit} />
                                                                    <label htmlFor="username"><i className="fa fa-user"></i>Enter Username</label>
                                                                </span>
                                                                <label className={this.state.errors.username?"error":""}>
                                                                    {this.state.errors.username?this.state.errors.username : ""}
                                                                </label>
                                                            </div>
                                                            <div className="form-field">
                                                                <span className="p-float-label">
                                                                    <InputText id="password" className={`${this.state.errors.password?"input-error ":""} form-control`} 
                                                                            type="password" value={this.state.password} onChange={(e) => this.setCredentials('password', e.target.value )} 
                                                                            onKeyUp={this.formSubmit} />
                                                                    <label htmlFor="password"><i className="fa fa-key"></i>Enter Password</label>
                                                                </span>
                                                                <label className={this.state.errors.password?"error":""}>
                                                                    {this.state.errors.password?this.state.errors.password : ""}
                                                                </label>
                                                            </div>
                                                            <div className="col-md-6 button-div loginbutton" style={{marginLeft: '12em'}}>                                                            
                                                                <button className="btn btn-primary" style={{marginLeft: '1em', width: '7em'}} id="systemloginbutton"
                                                                    disabled={Object.keys(this.state.validFields).length<2} 
                                                                    onClick={this.login}>Login</button>
                                                            </div>
                                                        </div>
                                                    }
                                                </div>
                                                 
                                                <div className="row form-footer">
                                                    <div className="col-md-6 login-form-link">
                                                        <a href={process.env.REACT_APP_REQUEST_ACCESS_URL} >Request access</a>
                                                    </div>
                                                    <div className="col-md-6 forget-paswd login-form-link">
                                                        <a href={process.env.REACT_APP_FORGOT_PASSWORD_URL} >Forgot Password?</a>
                                                    </div>
                                                    <div className="col-md-6 login-form-link">
                                                        <a href="/schedule">Public Schedule</a>
                                                    </div>
                                                </div>
                                                {this.state.error &&
                                                <div className="row error">
                                                    Unable to login. Please try with a different username and/or password.
                                                </div>
                                                }
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
            <CustomPageSpinner visible={this.state.showSpinner} />
            </>
        );
    }
}

Login.propTypes = {
  onLogin: PropTypes.func
}