import { Component } from 'react';
import { Dialog } from 'primereact/dialog';
import { Button } from 'primereact/button';
import Stations from '../../routes/Scheduling/Stations';
import PropTypes from 'prop-types';

import _ from 'lodash';

export default class StationEditor extends Component {
  constructor(props) {
    super(props);
    this.tmpRowData = [];
    this.isDelete = false;
    this.showDelete = false;
    this.previousValue = '';
    this.doCancel = true;
    this.state = {
      schedulingUnit: {},
      showDialog: false,
      dialogTitle: 'Station Group',
      missingStationFieldsErrors: [],
      stationGroup: [],
      customSelectedStations: []
    };
    this.formRules = {};
    this.eventHandling = this.eventHandling.bind(this);
  }

  isPopup() {
    return true;
  }

  /**
   * Init the date value if exists
   */
  async componentDidMount() {
    let tmpStationGroups = [];
    let tmpStationGroup = {};
    if (this.props.colDef.field.startsWith('gdef_')) {
      this.showDelete = true;
    }
    let rowSU = this.props.agGridReact.props.rowData[this.props.node.rowIndex];
    this.previousValue = rowSU[this.props.colDef.field];
    this.setState({ dialogTitle: this.props.colDef.headerName });
    this.props.colDef.suppressKeyboardEvent = params => {
      // return true (to suppress) if Tab key pressed
      const keyCode = params.event.keyCode;
      let gridShouldDoNothing = false;
      if (params.editing && (keyCode === 9)) {
        gridShouldDoNothing = true;
      }
      return gridShouldDoNothing;
    }

    if (this.previousValue && this.previousValue.length > 0) {
      let stationGroups = _.split(this.previousValue, "|");
      stationGroups.map(stationGroup => {
        tmpStationGroup = {};
        let sgValue = _.split(stationGroup, ":");
        if (sgValue && sgValue[0].length > 0) {
          let stationArray = _.split(sgValue[0], ",");

          tmpStationGroup['stations'] = stationArray;
          tmpStationGroup['max_nr_missing'] = sgValue[1];
          tmpStationGroups.push(tmpStationGroup);
        }

      })
      this.setState({
        stationGroup: tmpStationGroups,
        showDialog: true
      });
    } else {
      let defaultSGs = this.props.context.componentParent.state.defaultStationGroups;
      if (defaultSGs) {
        this.setState({
          stationGroup: defaultSGs,
          selectedStations: defaultSGs,
          showDialog: true
        });
      }
    }
  }

  async deleteStationGroup() {
    this.isDelete = true;
    this.setState({ showDialog: false });
  }

  async closeStationGroup() {
    this.isDelete = false;
    this.doCancel = false;
    this.updateStationGroup();
    this.setState({ showDialog: false });
  }

  async cancelStationGroup() {
    this.isDelete = false;
    this.doCancel = true;
    this.updateStationGroup();
    this.setState({ showDialog: false });
  }

  async updateStationGroup() {
    let stationValue = '';
    const station_groups = [];
    if (!this.isDelete) {
      if (!this.doCancel) {
        (this.state.selectedStations || []).forEach(key => {
          let station_group = {};
          const stations = this.state[key] ? this.state[key].stations : [];
          const max_nr_missing = parseInt(this.state[key] ? this.state[key].missing_StationFields ? this.state[key].missing_StationFields : this.state[key].max_nr_missing : 0);
          station_group = {
            stations,
            max_nr_missing
          };
          station_groups.push(station_group);
        });
        this.state.customSelectedStations.forEach(station => {
          station_groups.push({
            stations: station.stations,
            max_nr_missing: parseInt(station.max_nr_missing)
          });
        });
        if (station_groups) {
          station_groups.forEach(stationGroup => {
            stationValue += stationGroup.stations + ':' + stationGroup.max_nr_missing + "|";
          });
        }
      } else {
        stationValue = this.previousValue;
      }
    }
    await this.props.context.componentParent.updateCell(
      this.props.node.rowIndex, this.props.colDef.field, stationValue, true, true
    );
    this.setState({ showDialog: false });
  }

  onUpdateStations = (state, selectedStations, missingStationFieldsErrors, customSelectedStations) => {
    this.setState({
      ...state,
      selectedStations,
      missingStationFieldsErrors,
      customSelectedStations
    }, () => {
      this.setState({
        validForm: !missingStationFieldsErrors
      });
    });
  };

  /**
   * Function to handle tab key press for focus movement in the dialog.
   * @param {Key Event} e - Key code
   */
  eventHandling(e) {
    var key = e.which || e.keyCode;
    if (key === 9) {
      // Default focus is in dialog close button
      if (e.target.className.indexOf("p-dialog-titlebar-close") >= 0) {
        // Move focus to station group multiselect box
        document.getElementById("station-group-select").focus();
      }
      // By default focus is not moving out from multiselect in dialog. Multiselect input tags will not have id. 
      else if (!e.target.id) {
        // From station group multiselect move focus to Add Custom button
        if (e.target.parentElement.parentElement.id === 'station-group-select') {
          document.getElementById("add-custom-button").focus();
        }
        // From Custom stations move focus to Missing stations fields
        else if (e.target.parentElement.parentElement.id.indexOf('custom_stations_') >= 0) {
          let removeBtnIndex = e.target.parentElement.parentElement.id.replace("custom_stations_", "missingStation-");
          document.getElementById(removeBtnIndex).focus();
        }
      }
      // From Custom stations remove butto move focus to custom station multiselect
      else if (e.target.className.indexOf("custom-remove") >= 0) {
        let removeBtnIndex = e.target.id.replace("custom-remove-", "");
        document.getElementById("custom_stations_" + removeBtnIndex).focus();
      }
    }
  }

  render() {
    return (
      <div onKeyDown={this.eventHandling} >
        <Dialog header={'Station specification'} visible={this.state.showDialog} maximized={false}
          onHide={() => { this.updateStationGroup() }} inputId="confirm_dialog" className="stations_dialog"
          footer={<div>
            {this.showDelete &&
              <Button className="p-button-danger" icon="pi pi-trash" label="Clear All" onClick={() => { this.deleteStationGroup() }} />
            }
            <Button label="OK" icon="pi pi-check" onClick={() => { this.closeStationGroup() }}
              onKeyDown={(e) => { if (e.key === "Enter" || e.key === "") { this.closeStationGroup() } }}
              disabled={!this.state.validForm} style={{ width: '6em' }} />
            <Button className="p-button-danger" icon="pi pi-times" label="Cancel" onClick={() => { this.cancelStationGroup() }} />
          </div>
          } >
          <div className="ag-theme-balham" style={{ height: '90%', width: '1000px', paddingLeft: '20px' }}>
            <Stations
              taskName={this.state.dialogTitle}
              stationGroup={this.state.stationGroup}
              onUpdateStations={this.onUpdateStations.bind(this)}
              height={'30em'}
            />
          </div>
        </Dialog>
      </div>
    );
  }

}

StationEditor.propTypes = {
  colDef: PropTypes.shape({
    field: PropTypes.shape({
      startsWith: PropTypes.func
    }),
    headerName: PropTypes.any,
    suppressKeyboardEvent: PropTypes.any
  }),
  agGridReact: PropTypes.shape({
    props: PropTypes.shape({
      rowData: PropTypes.any
    })
  }),
  node: PropTypes.shape({
    rowIndex: PropTypes.any
  }),
  context: PropTypes.shape({
    componentParent: PropTypes.shape({
      state: PropTypes.shape({
        defaultStationGroups: PropTypes.any
      }),
      updateCell: PropTypes.func
    })
  })
}