import PropTypes from 'prop-types';
import { Component } from 'react';
import ProjectService from '../../services/project.service';
import ViewTable from '../../components/ViewTable/ViewTable';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import CycleService from '../../services/cycle.service';
import UtilService from '../../services/util.service';
import AuthUtil from '../../utils/auth.util';
import _ from 'lodash';

import { CustomDialog } from '../../layout/components/CustomDialog';
import { Column } from 'primereact/column';
import { DataTable } from 'primereact/datatable';
import { appGrowl } from '../../layout/components/AppGrowl';
import TopProgressBar from '../../layout/components/TopProgressBar';
import ProjectStatusDialog from './projectStatusDialog'
export class ProjectList extends Component {
    lsTableName = 'project_list';
    lsKeySortColumn = "projectSortData";
    defaultSortColumn= [{ id: "Name / Project Code", desc: false }];
    constructor(props) {
        super(props)
        this.state = {
            userrole: {
                userRolePermission: {}
            },
            projectlist: [],
            showStatusUpdateDialog: false,
            defaultcolumns: [{
                name: "Name / Project Code",
                project_state_value: {
                    name: "Status",
                    filter: "select",
                    showTag: true,
                    tagClass: 'proj-$dynamic$' //$dynamic$ will be replace with cell value
                },
                project_category_value: {
                    name: "Category of Project",
                    filter: "select"
                },
                description: "Description",              
                archive_location_label: {
                    name: "LTA Storage Location",
                    filter: "select"
                },
                cycles_ids: {
                    name: "Cycles",
                    filter: "select"
                },
                'LOFAR Observing Time': {
                    name: "Observing time (Hrs)",
                    filter: "range"
                },
                'LOFAR Observing Time prio A': {
                    name: "Observing time prio A (Hrs)",
                    filter: "range"
                },
                'LOFAR Observing Time prio B': {
                    name: "Observing time prio B (Hrs)",
                    filter: "range"
                },
                'CEP Processing Time': {
                    name: "Processing time (Hrs)",
                    filter: "range"
                },
                'LTA Storage': {
                    name: "LTA storage (TB)",
                    filter: "range"
                },
                'Number of triggers': {
                    name: "Number of Triggers",
                    filter: "range"
                },
                friend: {
                    name: "Friends"
                },
                myRoles: {
                    name: "My Roles"
                }
            }],
            optionalcolumns: [{
                auto_ingest: {
                    name: "Ingest Automatically",
                    filter: "switch",
                },
                rank: {
                    name: "Project Priority",
                    filter: "range"
                },
                trigger_priority: {
                    name: "Trigger Priority",
                    filter: "range"
                },
                period_category_value: {
                    name: "Category of Period",
                    filter: "select"
                },
                can_trigger: {
                    name: "Trigger Allowed",
                    filter: "switch"
                },
                auto_pin: {
                    name: "Prevent automatic deletion after ingest",
                    filter: "switch"
                },
                actionpath: "actionpath"
            }],
            columnclassname: [{
                "Observing time (Hrs)": "filter-input-50",
                "Observing time prio A (Hrs)": "filter-input-50",
                "Observing time prio B (Hrs)": "filter-input-50",
                "Processing time (Hrs)": "filter-input-50",
                "LTA storage (TB)": "filter-input-50",
                "Status": "filter-input-50",
                "Category of Project": "filter-input-100",
                "Trigger Allowed": "filter-input-50",
                "Number of Triggers": "filter-input-50",
                "Project Priority": "filter-input-50",
                "Trigger Priority": "filter-input-50",
                "Category of Period": "filter-input-50",
                "Cycles": "filter-input-100",
                "LTA Storage Location": "filter-input-100",
                "LTA Storage Path": "filter-input-100"
            }],
            defaultSortColumn: [{ id: "Name / Project Code", desc: false }],
            columnOrder: ["Action", "Name / Project Code", "Status", "Category of Project", "Description",
                            "LTA Storage Location", "Ingest Automatically", "Project Priority",
                            "Trigger Priority", "Category of Period", "Cycles", "Trigger Allowed",
                            "Observing time (Hrs)", "Observing time prio A (Hrs)", "Observing time prio B (Hrs)",
                            "Processing time (Hrs)", "LTA storage (TB)", "Number of Triggers",
                            "Prevent automatic deletion after ingest",
                            ],
            isprocessed: false,
            isLoading: true,
            fetchingFriendOfProj: false,            
        }
        this.updatedProjList = [];
        this.selectedRows = [];
        this.statusOptions = [];
        this.pageUpdated = true;
        this.myRoles = {};
        this.getPopulatedProjectList = this.getPopulatedProjectList.bind(this);
        this.toggleBySorting = this.toggleBySorting.bind(this);
        this.setToggleBySorting = this.setToggleBySorting.bind(this);
        this.onRowSelection = this.onRowSelection.bind(this);
        this.showStatusChangeDialog = this.showStatusChangeDialog.bind(this);
        this.confirmStatusChange  = this.confirmStatusChange.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
        this.submitStatusChange = this.submitStatusChange.bind(this);
        this.getFilterOptions = this.getFilterOptions.bind(this);
        this.setToggleBySorting();
    }

    async getPopulatedProjectList(cycleId) {
        const permission = await AuthUtil.getUserRolePermission();
        this.myRoles = await ProjectService.getMyRoles();
        this.setState({userrole: permission});
        const projectRoles = Object.fromEntries(
            Object.entries(this.myRoles.project_roles).map(([project, roles]) => [project.toLowerCase(), roles])
        );
        this.myRoles.project_roles = projectRoles;
        Promise.all([ProjectService.getFileSystem(), ProjectService.getCluster()]).then(async (response) => {
            const options = {};
            if (response.length === 2 && response[0] && response[1]) {
                response[0].map(fileSystem => {
                    const cluster = response[1].filter(clusterObj => { return (clusterObj.id === fileSystem.cluster_id && clusterObj.archive_site); });
                    if (cluster.length) {
                        fileSystem.label = `${cluster[0].name} - ${fileSystem.name}`
                        options[fileSystem.url] = fileSystem;
                    }
                    return fileSystem;
                });
            }
            let projects = [];
            if (cycleId) {
                projects = await CycleService.getProjectsByCycle(cycleId);
            } else {
                projects = await ProjectService.getProjectList(true);
            }
            let list = [];
            for(let project of projects) {
                project.friend = 'Loading...';
                list.push(project);
            }
            const projectArchiveLocation = await ProjectService.getAllProjectQuotaArchiveLocation();
            projects = await ProjectService.getUpdatedProjectQuota(projects);
            this.setState({
                projectlist: projects,
                fetchingFriendOfProj: true,
                isprocessed: true,
                isLoading: false,
                ltaStorage: options                
            });
            list = projects;
            this.updatedProjList = [];
            projects.forEach((project, index) => {
                ProjectService.getFriendsOfProject(project.name).then(async (response) => {
                    project.friend = response.join(", ");
                    project.myRoles = this.myRoles.project_roles[project.name.toLowerCase()]?.join(", ");
                    project.quota.map(quota => {
                        let projectArchive = _.find(projectArchiveLocation.results, ['project_quota', quota]);
                        if(projectArchive) {
                            project.archive_location_label = (options[projectArchive.archive_location] || {}).label;
                        }
                        return quota;
                    });
                    list[index] = project;
                    this.updatedProjList.push(project);
                    if (this.updatedProjList.length > 0 && this.updatedProjList.length === list.length) {
                        this.setState({
                            projectlist: list,
                            isprocessed: true,
                            isLoading: false,
                            fetchingFriendOfProj: false
                        });
                    }   else {
                        this.setState({
                            projectlist: list,
                        });
                    }
                });
            });
        });
    }

    async componentDidMount() {
       //this.getUserRolePermission();
        // Show Project for the Cycle, This request will be coming from Cycle View. Otherwise it is consider as normal Project List.
        this.pageUpdated = true;
        let cycle = this.props.cycle;
        this.getPopulatedProjectList(cycle);
        this.setToggleBySorting();
        this.statusOptions = await ProjectService.getProjectStates();
    }

    setToggleBySorting() {
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if(sortData){
            if(Object.prototype.toString.call(sortData) === '[object Array]'){
                this.defaultSortColumn = sortData;
            }
            else{
                this.defaultSortColumn = [{...sortData}];
            }
        }else{
            this.defaultSortColumn =  [{ id: "Name / Project Code", desc: false }];
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] })        
    }

    toggleBySorting(sortData) {
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: sortData });
    }

    /**
     * Content for confirmation dialog
     * @returns Content to be dispalyed in the confirmation dialog
     */
    getSUDialogContent = () => {
        let changedData = [];
        for (const row of this.selectedRows) {
            let changedRow = {}
            changedRow.project_state_value = this.state.changedStatus;
            changedRow.name = row.name;
            changedData.push(changedRow);
        }
        return (
                <div style={{ marginTop: '1em' }}>
                    <b>Do you want to change the status of selected Project(s)?</b>
                    <div style={{ marginTop: '1em' }}>                 
                        <DataTable value={changedData} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="name" header="Project Name"></Column>
                            <Column field="project_state_value" header="Status"></Column>
                        </DataTable>                  
                    </div>
                </div>
        )
    }

    /**
     * 
     * @returns Content to be displayed in dialog after updating the staus in backend
     */
    getSUResponseDialogContent = () => {
        let changedData = [];
        for (const row of this.state.statusChangeResponse) {
            changedData.push(row);
        }
        return (
                <div style={{ marginTop: '1em' }}>
                    <b></b>
                    <div style={{ marginTop: '1em' }}>                 
                        <DataTable value={this.state.statusChangeResponse} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="name" header="Project Name"></Column>
                            <Column field="status" header="Project Status"></Column>
                            <Column field="message" header="Update Status"></Column>
                        </DataTable>                  
                    </div>
                </div>
        )
    }

    onRowSelection(selectedRows) {
        this.selectedRows = selectedRows;
    }

    /**
     * Show the available status options to change
     */
    showStatusChangeDialog() {
        if(this.selectedRows.length>0) {
            this.setState({ showStatusUpdateDialog: !this.state.showStatusUpdateDialog, changedStatus: null });
        }
        else {
            appGrowl.show({severity: 'info',summary: 'Select Row', detail: 'Please select one or more Project(s)'});
        }
    }

    /**
     * Show confirmation dialog to update status change
     */
    confirmStatusChange () {
        let dialog = {};
        dialog.type = "confirmation";
        dialog.header = `Confirm Project(s) Status Change`;
        dialog.content = this.getSUDialogContent;
        dialog.actions = [{
            id: 'yes', title: 'Yes', callback: () => {
                this.submitStatusChange();
            }
        },
        { id: 'no', title: 'No', callback: () => {
            this.closeDialog()
        } }];
        dialog.onSubmit = () => {};
        dialog.width = '25vw';
        dialog.showIcon = false;
        dialog.dialogVisible = true;
        this.setState({dialog: dialog})          
    }

    closeDialog () {
        let dialog = this.state.dialog;
        dialog.dialogVisible = false
        this.setState({dialog: dialog});
        return false;
    }
    
     /**
     * Updating status in backend on confirmation and displaying response of the updation
     */
    async submitStatusChange () {
        let statusChangeResponse = []
        for (const row of this.selectedRows) {
            row.project_state = this.state.changedStatus.url
            row.project_state_value = this.state.changedStatus;
            let response = await ProjectService.updateProject(row.name, row)
            if(response.isUpdated) {
                statusChangeResponse.push({
                    name: response.name,
                    status: response.project_state_value,
                    message: 'Success'
                })
            }
            else {
                statusChangeResponse.push({
                    name: response.name,
                    status: response.project_state_value,
                    message: 'Failed'
                })
                appGrowl.show({ severity: 'error', summary: 'Failed to update project status'});
            }
        }
        this.setState({statusChangeResponse: statusChangeResponse})
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = `Project(s) Status Update Info`;
        dialog.content = this.getSUResponseDialogContent;
        dialog.actions = [{
            id: 'yes', title: 'Close', callback: () => {
                this.closeDialog();
                this.setState({showStatusUpdateDialog: false});
            }
        }];
        dialog.onSubmit = () => {
            this.closeDialog();
            this.setState({showStatusUpdateDialog: false});
        }
        dialog.width = '30vw';
        dialog.showIcon = false;
        dialog.dialogVisible = true; 
        this.setState({dialog: dialog, changedStatus: {}})
    }

    /**
    * Get Option-list values for Select Dropdown filter in 'Viewtable'
    * @param {String} id : Column id
    * @returns 
    */
    getFilterOptions(id) {
        let options = [];
        if(id && id === 'Status') {
            options = (_.map(this.statusOptions, status => status.value)).sort((a, b) => a.localeCompare(b))
        }
        return options;
      }

   

    prepareSaveProjectStatus  = (projectStatus) => {
        this.setState({ changedStatus: projectStatus },() =>this.confirmStatusChange());
    }

    gotoAdd = () =>{
        
       this.props. history.push("/project/create") ;

    }
    render() {

        return (
            <>
                { (this.props.cycle) ?
                    <>
                    </>
                    :
                    <PageHeader location={this.props.location} title={'Project - List'} className="defaultpageHeader"
                        actions={[{icon: 'pi pi-briefcase',title: this.state.userrole?.userRolePermission.project?.edit?'Update Project(s) Status':"Don't have permission to Update Status", 
                        disabled: this.state.userrole?.userRolePermission.project?!this.state.userrole.userRolePermission.project.edit:true,
                        type: 'buttonv2', actOn: 'click', props: { callback: this.showStatusChangeDialog }},
                        { 
                             icon: 'pi-plus',
                             type: 'buttonv2',
                             title: this.state.userrole?.userRolePermission.project?.create?'Click to Add Project':"Don't have permission to add new Project", 
                             disabled: this.state.userrole?.userRolePermission?.project?!this.state.userrole.userRolePermission.project.create:true,
                             actOn: 'click', props:{ callback:this.gotoAdd }}]}/> 
                    
                }
                {this.state.isLoading ? <AppLoader /> : (this.state.isprocessed && this.state.projectlist.length > 0) ?
                    <div>
                       {this.state.fetchingFriendOfProj &&  <TopProgressBar />}
                        <ViewTable
                            data={this.state.projectlist}
                            defaultcolumns={this.state.defaultcolumns}
                            optionalcolumns={this.state.optionalcolumns}
                            columnclassname={this.state.columnclassname}
                            columnOrders={this.state.columnOrder}
                            defaultSortColumn={this.defaultSortColumn}
                            showaction={true}
                            paths={this.state.paths}
                            keyaccessor="name"
                            unittest={this.state.unittest}
                            tablename={this.lsTableName}
                            allowRowSelection={true}
                            onRowSelection={this.onRowSelection}
                            toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                            lsKeySortColumn={this.lsKeySortColumn}
                            pageUpdated={this.pageUpdated}
                            storeFilter={true}
                            showFilterOption={this.getFilterOptions} //Callback function to provide inputs for option-list in Select Dropdown filter
                        />   
                    </div>                
                    : <div>No project found </div>
                }
                {this.state.showStatusUpdateDialog &&
                <div>
                    <ProjectStatusDialog 
                        statusOptions= {this.statusOptions}
                        visible={this.state.showStatusUpdateDialog}
                        onHide={ () => { this.setState({ showStatusUpdateDialog: false }) }}
                        onCancel={ () => { this.setState({ showStatusUpdateDialog: false }) }}
                        onSave={ (projectStatus) => {this.prepareSaveProjectStatus(projectStatus)}} 
                        /> 

                    <CustomDialog type="confirmation" visible={this.state.dialog && this.state.dialog.dialogVisible}
                        header={this.state.dialog?.header} message={this.state.dialog?.detail} actions={this.state.dialog?.actions}
                        content={this.state.dialog?.content} width={this.state.dialog?.width} showIcon={this.state.dialog?.showIcon}
                        onClose={this.closeDialog} onCancel={this.closeDialog}/>
                </div>
                }
            </>
        )
    }

}

ProjectList.propTypes = {
  cycle: PropTypes.any,
  history: PropTypes.object,
  location: PropTypes.oneOfType([PropTypes.object, PropTypes.string])
}
export default ProjectList;