import React from "react";
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import PropTypes from 'prop-types';

export default function CycleTelescopeTime(props) {
    let { data } = props;

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    const getBarData = () => {
        let reportData = data;
        reportData = reportData?.slice(0, -1);
        return {
            labels: _.map(reportData, 'cycle'),
            datasets: [

                {
                    label: 'Regular_DDT_Comm',
                    data: _.map(reportData, 'reg_ddt_comm'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                },
                {
                    label: 'Filler',
                    data: _.map(reportData, 'filler'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,
                },
                {
                    label: 'Failed',
                    data: _.map(reportData, 'failed'),
                    backgroundColor: '#A5A5A5',
                    barThickness: 75,
                },
                {
                    label: 'System/Idle',
                    data: _.map(reportData, 'idle'),
                    backgroundColor: '#FFC000',
                    barThickness: 75,
                }
            ]
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    const getBarOptions = () => {
        return {
            indexAxis: 'x',
            // Elements options apply to all of the options unless overridden in a dataset
            // In this case, we are setting the border of each horizontal bar to be 1px wide
            elements: {
                bar: {
                    borderWidth: 1,
                },
            },
            scales: {
                x: {
                    stacked: true,
                    categoryPercentage: 1.0,
                    barPercentage: 1.0
                },
                y: {
                    stacked: true,
                    max: 100,
                }
            },
            responsive: true,
            plugins: {
                legend: {
                    position: 'bottom',
                },
                title: {
                    display: true,
                    text: 'Telescope time distribution',
                },
            }
        };
    }



    return (
        <React.Fragment>
            <div id={`cycles-telescope-time`}
                style={{ paddingTop: "10px", paddingBottom: "10px" }}>
                <Bar data={getBarData()} options={getBarOptions()} width="50%" height="20" />
            </div>
            <div className="ttd-details" id={`cycle-ttd-details`}>
                <DataTable value={data} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em', textAlign: 'center' }}>
                    <Column field="cycle" header="Cycle Code"></Column>
                    <Column field="reg_ddt_comm" header="Regular/DDT/Comm (%)"></Column>
                    <Column field="filler" header="Filler (%)"></Column>
                    <Column field="failed" header="Failed (%)"></Column>
                    <Column field="idle" header="System/Idle (%)"></Column>
                </DataTable>
            </div>
        </React.Fragment>
    )
}


CycleTelescopeTime.propTypes = {
  data: PropTypes.array
}