import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { Redirect } from 'react-router-dom'
import moment from 'moment';
import Jeditor from '../../components/JSONEditor/JEditor';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';

import UIConstants from '../../utils/ui.constants';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { appGrowl } from '../../layout/components/AppGrowl';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import ReservationService from '../../services/reservation.service';
import UnitConverter from '../../utils/unit.converter';
import AuthStore from '../../authenticate/auth.store';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import Websocket from 'react-websocket';
import ParserUtility from '../../utils/parser.utility';

export class ReservationView extends Component {
    constructor(props) {
        super(props);
        this.state = {
            isLoading: true,
            confirmDialogVisible: false,
            userrole: AuthStore.getState(),
            actions: []
        };
        this.dialogType = "confirmation";
        this.dialogHeader = "";
        this.dialogMsg = "";
        this.dialogContent = "";
        this.callBackFunction = "";
        this.dialogWidth = '40vw';
        this.onClose = this.close;
        this.onCancel =this.close;
        this.deleteReservation = this.deleteReservation.bind(this);
        this.showConfirmation = this.showConfirmation.bind(this);
        this.cancelView = this.cancelView.bind(this);
        this.close = this.close.bind(this);
        this.getDialogContent = this.getDialogContent.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.handleData = this.handleData.bind(this);
        this.refreshReservation = this.refreshReservation.bind(this);
        this.getReservationDetails = this.getReservationDetails.bind(this);
        this.prepareActionMenus = this.prepareActionMenus.bind(this);
        
        if (this.props.match?.params?.id) {
            this.state.taskId  = this.props.match.params.id;
        }
        if (this.props.match?.params?.type) {
            this.state.taskType = this.props.match.params.type;
        }
        
    }

    async componentDidMount() {
        const permission = await AuthUtil.getUserRolePermission();
        this.setState({userrole: permission})
        const reserId = this.props.match?this.props.match?.params?.id: null;
        this.getReservationDetails(reserId);
    }

     
    /**
     * To get the Reservation details from the backend using the service
     * @param {number} Reservation Id
     */
    getReservationDetails(id) {
        if (id) {
            ReservationService.getReservation(id)
            .then((reservation) => {
                if (reservation) {
                    ReservationService.getReservationTemplate(reservation.specifications_template_id)
                    .then((reservationTemplate) => {
                        reservationTemplate.schema = ParserUtility.updateReservationPropsOrder(reservationTemplate.schema, false);
                        this.setState({redirect: null, reservation: reservation, isLoading: false, reservationTemplate: reservationTemplate});
                        this.prepareActionMenus();
                    });
                }   else {
                    this.setState({redirect: "/not-found"});
                }
            });
        }   else {
            this.setState({redirect: "/not-found"});
        }
    }

    /**
     * Show confirmation dialog
     */
    showConfirmation() {
        this.dialogType = "confirmation";
        this.dialogHeader = "Confirm to Delete Reservation";
        this.dialogMsg = "Do you want to delete this Reservation?";
        this.dialogWidth = '55vw';
        this.dialogContent = this.getDialogContent;
        this.callBackFunction = this.deleteReservation;
        this.onClose = this.close;
        this.onCancel =this.close;
        this.setState({confirmDialogVisible: true});
    }

    /**
     * Prepare Reservation details to show on confirmation dialog
     */
    getDialogContent() {
        let reservation = this.state.reservation;
        reservation['start_time'] = (reservation['start_time'] && reservation['start_time'] !== 'Unknown' )?moment.utc(reservation['start_time']).format(UIConstants.CALENDAR_DATETIME_FORMAT): 'Unknown';
        reservation['stop_time'] = (reservation['stop_time'] && reservation['stop_time'] !== 'Unknown' )?moment.utc(reservation['stop_time']).format(UIConstants.CALENDAR_DATETIME_FORMAT): 'Unknown';
        return  <> 
                   <DataTable value={[reservation]} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                        <Column field="id" header="Reservation Id"></Column>
                        <Column field="name" header="Name"></Column>
                        <Column field="start_time" header="From Date"></Column>
                        <Column field="stop_time" header="To Date"></Column>
                    </DataTable>
                </>
    }

    close() {
        this.setState({confirmDialogVisible: false});
    }

    cancelView(){
        this.props.history.length>1?this.props.history.goBack():this.props.history.push(`/task`);
    }

    /**
     * Delete Reservation
     */
    async deleteReservation() {
        let hasError = false;
        const reserId = this.props.match?this.props.match.params.id: null;
        if(!await ReservationService.deleteReservation(reserId)){
            hasError = true;
        }
        if(hasError){
            appGrowl.show({severity: 'error', summary: 'error', detail: 'Error while deleting Reservation'});
            this.setState({confirmDialogVisible: false});
        }   else {
            appGrowl.show({severity: 'success', summary: 'Success', detail: 'Reservation deleted successfully'});
            this.setState({confirmDialogVisible: false});
            this.setState({redirect: `/reservation/list`});
        }
    }

    /**
     * Prepare action menu
     * @param {*} disableUpdateAction : true - relevant button will be disabled
     */
    prepareActionMenus(disableUpdateAction) {
        const permissions = this.state.userrole.userRolePermission.reservation;
        let actions = [ ];
        actions.push({ icon: 'fa-edit', 
                        title: permissions?.edit?disableUpdateAction === undefined?'Click to Edit Reservation':'This Reservation is deleted by another user.': "Don't have permission to edit",
                        disabled: disableUpdateAction === undefined?permissions?.edit? !permissions?.edit : true: disableUpdateAction,
                        props : { pathname:`/reservation/edit/${this.state.reservation?this.state.reservation.id:null}`}}); 
        actions.push({ icon: 'fa fa-trash',
                        title: permissions?.delete?disableUpdateAction === undefined?'Click to Delete Reservation':'This Reservation is deleted by another user.': "Don't have permission to delete",
                        disabled: disableUpdateAction === undefined?!permissions?.delete: disableUpdateAction,
                        type: 'button',  actOn: 'click', props:{ callback: this.showConfirmation}});
        actions.push({  icon: 'fa-window-close', link: this.props.history.goBack,
                        title:'Click to Close Reservation',  props:{ callback: this.cancelView }});
        this.setState({actions: actions})
    }

    /**
     * Function to call wnen websocket is connected
     */
     onConnect() {
        try{
            const userDets = localStorage.getItem("user");
            if (userDets) {
                this.websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }

    /**
     * Function to call when websocket is disconnected
     */
    onDisconnect() {
        console.log("WS Closed")
    }

    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    handleData(data) {
        if (data) {
            console.log('received websocket data:', data)
            const jsonData = JSON.parse(data);
            this.refreshReservation(jsonData); 
        }
    }
    
    /**
     * Refresh Reservation parameter
     * @param {*} jsonData 
     */
    refreshReservation(jsonData) {
        if (jsonData.object_type === 'reservation' && this.state.reservation.id === jsonData.object_details.id) { 
            if (jsonData.action === 'update') {
                this.getReservationDetails(jsonData.object_details.id);
            }   else if (jsonData.action === 'delete') {
                this.prepareActionMenus(true);
                appGrowl.show({severity: 'error', summary: 'error', sticky: true, detail: `This Reservation [${this.state.reservation.name}] is deleted by another user and will not be available for further action.`});
            }   
        } 
    } 

    render() {
        const permissions = this.state.userrole.userRolePermission.reservation;
        if (this.state.redirect) {
            return <Redirect to={ {pathname: this.state.redirect} }></Redirect>
        }
        let jeditor = null;
        if (this.state.reservationTemplate) {
            if (this.state.reservation.specifications_doc?.$id) {
                delete this.state.reservation.specifications_doc.$id;
                delete this.state.reservation.specifications_doc.$schema;
            }
            jeditor = React.createElement(Jeditor, {title: "Reservation Parameters", 
                                                        schema: this.state.reservationTemplate.schema,
                                                        initValue: this.state.reservation.specifications_doc,
                                                        disabled: true,
                                                    });
        }

        
        return (
            <React.Fragment>
                {permissions?.list ?
                <>
                <PageHeader location={this.props.location} title={'Reservation – Details'} actions={this.state.actions}/>
                { this.state.isLoading? <AppLoader /> : this.state.reservation &&
                    <React.Fragment>
                        <div className="main-content">
                        <div className="p-grid">
                            <label className="col-lg-2 col-md-2 col-sm-12">Name</label>
                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.reservation.name}</span>
                            <label className="col-lg-2 col-md-2 col-sm-12">Description</label>
                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.reservation.description}</span>
                        </div>
                        <div className="p-grid">
                            <label className="col-lg-2 col-md-2 col-sm-12">Start Time</label>
                            <span className="col-lg-4 col-md-4 col-sm-12">{moment.utc(this.state.reservation.start_time).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                            <label className="col-lg-2 col-md-2 col-sm-12">End Time</label>
                            <span className="col-lg-4 col-md-4 col-sm-12">{(this.state.reservation.stop_time && this.state.reservation.stop_time !== 'Unknown')?moment.utc(this.state.reservation.stop_time).format(UIConstants.CALENDAR_DATETIME_FORMAT): 'Unknown'}</span>
                        </div>
                        <div className="p-grid">
                            <label className="col-lg-2 col-md-2 col-sm-12">Project</label>
                            <span className="col-lg-4 col-md-4 col-sm-12">{(this.state.reservation.project_id)?this.state.reservation.project_id:''}</span>
                            <label className="col-lg-2 col-md-2 col-sm-12">Duration (Days HH:mm:ss)</label>
                            <span className="col-lg-4 col-md-4 col-sm-12">{(this.state.reservation.duration)?UnitConverter.getSecsToDDHHmmss(this.state.reservation.duration):'Unknown'}</span>
                        </div>
                       
                        <div className="p-fluid">
                            <div className="p-grid"><div className="p-col-12" style={{width: "100%"}}>
                                {this.state.reservationTemplate?jeditor:""}
                            </div></div>
                        </div>
                        </div>
                    </React.Fragment>
                }
                 <CustomDialog type={this.dialogType} visible={this.state.confirmDialogVisible} width={this.dialogWidth}
                    header={this.dialogHeader} message={this.dialogMsg} 
                    content={this.dialogContent} onClose={this.onClose} 
                    actions={ [ {id:"yes", title: 'Delete', callback: this.callBackFunction, className:'act-btn-dispose'},
                        {id:"no", title: 'Cancel', className:'act-btn-cancel', callback: this.onCancel} ]}  >
                </CustomDialog>
                {!this.state.isLoading &&
                    <Websocket ref={websocket => this.websocket = websocket} url={process.env.REACT_APP_WEBSOCKET_URL} 
                        onOpen={this.onConnect} onMessage={this.handleData} onClose={this.onDisconnect} />}
                </>: <AccessDenied/>}
            </React.Fragment>
        );
    }
}

ReservationView.propTypes = {
    match: PropTypes.object,
  history: PropTypes.object,
  location: PropTypes.oneOfType([PropTypes.object, PropTypes.string])
}