import PropTypes from 'prop-types';
import React, { useState } from 'react';

import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
export function SchedulingUnitListLite(props) {
    const {
        adjust,
        className,
        blueprints
    } = props

    let topAdjust = adjust?.height;

    const fetchTableData = () => {
        try {
            if (!blueprints) {
                let cachedBluePrints   = sessionStorage.getItem("blueprints")
                let parsed = JSON.parse(cachedBluePrints)
                setSlimBluePrints(parsed)
            } else
            {
                setSlimBluePrints(blueprints)
            }
            
            
        }
        catch (ex) {
            console.log("Unable to read session")
        }

    }
    React.useEffect(() => {
        if (blueprints) setSlimBluePrints(blueprints)
    }, [blueprints]);

    const formatDate = (dateString) => {
        if (!dateString) return;
        const date = new Date(dateString);
        return date.toISOString().slice(0, 19).replace('T', ' ');
    };
    React.useEffect(() => {
        topAdjust = adjust?.height;
        setTableHeight(window.innerHeight - topAdjust);

    }, [adjust]);
    React.useEffect(() => {
        fetchTableData();
        const handleResize = () => {
            setTableHeight(window.innerHeight - topAdjust);
        };

        window.addEventListener('resize', handleResize);
        return () => {
            window.removeEventListener('resize', handleResize);
        }
    }, []);

    const [slimBluePrints, setSlimBluePrints] = useState([])
    const [expandedRows, setExpandedRows] = useState(null);
    const [tableHeight, setTableHeight] = useState(window.innerHeight - topAdjust);
    /**
     * Fetch data from server side - while doing pagination, filtering, sorting
     * @param {Table State} Table props state 
     * @returns 
     */
    const allowExpansion = (rowData) => {
        return rowData.task_blueprints.length > 0;
    };

    const process_start_time_Template = (date) => {
        return formatDate(date?.process_start_time);
    };
    const process_stop_Template = (date) => {
        return formatDate(date?.process_stop_time);
    };
    const on_sky_start_Template = (date) => {
        return formatDate(date?.on_sky_start_time);
    };
    const on_sky_stop_timeTemplate = (date) => {
        return formatDate(date?.on_sky_stop_time);
    };

    const linkwithId = (data) => {
        return (
            <><a href={'/schedulingunit/view/blueprint/' + data.id} target="_blank" rel="noreferrer"> <i className='pi pi-link'></i>&nbsp;{data.id}</a> </>
        )
    }
    const linkTaskwithId = (data) => {
        return (
            <><a href={'/task/view/blueprint/' + data.id} target="_blank" rel="noreferrer"> <i className='pi pi-link'></i>&nbsp;{data.id}</a> </>
        )
    }


    const statusClass = (data) => {
        let statusclass = "su-" + data.status;
        return statusclass
    }

 



    const rowExpansionTemplate = (data) => {
        return (
            <div className="p-3">
                <h5>Task for {data.name}</h5>
                <DataTable value={data.task_blueprints}
                    reorderableColumns
                    stateStorage='local'
                    stateKey='DataTableSchedulingUnitListLiteTasks'
                    resizableColumns
                >
                    <Column sortable field="id" header="Id"  style={{ width: '75px' }} body={linkTaskwithId} bodyClassName={statusClass}></Column>
                    <Column sortable field="name" header="name" style={{ width: '100px' }} bodyClassName={statusClass}></Column>
                    <Column sortable field="status" header="Status" style={{ width: '100px' }} bodyClassName={statusClass} ></Column>
                    <Column sortable field="process_start_time" header="Process Start Time" style={{ width: '100px' }} body={process_start_time_Template} bodyClassName={statusClass}></Column>
                    <Column sortable field="process_stop_time" header="Process Stop time" style={{ width: '100px' }} body={process_stop_Template} bodyClassName={statusClass}></Column>
                    <Column sortable field="on_sky_start_time" header="on_sky_start_time" style={{ width: '100px' }} body={on_sky_start_Template}  bodyClassName={statusClass}></Column>
                    <Column sortable field="on_sky_stop_time" header="on_sky_stop_time" style={{ width: '100px' }} body={on_sky_stop_timeTemplate} bodyClassName={statusClass}></Column>
                    <Column sortable field="task_type" header="task type" style={{ width: '100px' }} bodyClassName={statusClass} ></Column>
                    <Column sortable field="duration" header="duration"  bodyClassName={statusClass}></Column>
                    <Column sortable field="description" header="Description" style={{ maxwidth: '300px' }} bodyClassName={statusClass}></Column>
                    <Column sortable field="unschedulable_reason" header="unschedulable reason" style={{ width: '300px' }} bodyClassName={statusClass}></Column>
                    <Column sortable field="error_reason" header="error reason" style={{ width: '300px' }} bodyClassName={statusClass} ></Column>
                </DataTable>
                <table>
                    <tbody>
                    <tr><td><b>Stations specified </b> </td><td> {data?.specified_stations?.join(", ")}</td></tr>
                    <tr><td><b>Stations used </b> </td><td> {data?.used_stations?.join(", ")}</td></tr>
                  {data?.missing_stations?.length>0  &&  (<tr><td><b>Stations missing </b></td><td>{data?.missing_stations?.join(", ")}</td></tr>) }
                  </tbody>
                </table>            
                
                
            </div>
        );
    };
    return (


        <DataTable
            stateStorage='local'
            stateKey='DataTableSchedulingUnitListLite'
            value={slimBluePrints}
            scrollable
            className={"wrapper" + className}
            scrollHeight={`${tableHeight}px`}
            reorderableColumns
            resizableColumns
            expandedRows={expandedRows} onRowToggle={(e) => setExpandedRows(e.data)}
            rowExpansionTemplate={rowExpansionTemplate}
            dataKey="id" tableStyle={{ minWidth: '60rem' }}

        >
            <Column expander={allowExpansion} style={{ width: '5rem' }} bodyClassName={statusClass} />
            <Column sortable key="id" field="id" header="id" body={linkwithId} bodyClassName={statusClass}></Column>
            <Column sortable key="name" field="name" header="Name"  bodyClassName={statusClass}></Column>
            <Column sortable key="project" field="project" header="project" bodyClassName={statusClass}/>
            <Column sortable key="description" field="description" header="Description" className="wrapper" bodyClassName={statusClass}/>
            <Column sortable key="status" field="status" header="status" bodyClassName={statusClass} />
            <Column sortable key="unschedulable_reason" field="unschedulable_reason" header="Unschedulable Reason" bodyClassName={statusClass} />
            <Column sortable key="error_reason" field="error_reason" header="Error reason" bodyClassName={statusClass}/>
            <Column sortable key="priority_queue" field="priority_queue" header="Priority Queue" style={{ width: '50px' }} bodyClassName={statusClass}/>
            <Column sortable key="scheduling_constraints_template_id" field="scheduling_constraints_template_id" header="T.Id" dataType="numeric" bodyClassName={statusClass}/>
            <Column sortable key="process_start_time" field="process_start_time" header="Process start time" body={process_start_time_Template} bodyClassName={statusClass}/>
            <Column sortable key="process_stop_time" field="process_stop_time" header="Process stop time" body={process_stop_Template} bodyClassName={statusClass}/>
            <Column sortable key="on_sky_start_time" field="on_sky_start_time" header="Sky start time" body={on_sky_start_Template}bodyClassName={statusClass} />
            <Column sortable key="on_sky_stop_time" field="on_sky_stop_time" header="Sky stop time By" dataType="date" body={on_sky_stop_timeTemplate}bodyClassName={statusClass} />
            <Column sortable key="duration" field="duration" header="duration" type="text" bodyClassName={statusClass}/>
            <Column sortable key="on_sky_duration" field="on_sky_duration" header="on sky duration" bodyClassName={statusClass}/>
            <Column sortable key="antenna_set" field="antenna_set" header="antenna set" bodyClassName={statusClass} />
        </DataTable>
    );
}

SchedulingUnitListLite.propTypes = {
  adjust: PropTypes.shape({
    height: PropTypes.any
  }),
  className: PropTypes.any,
  blueprints: PropTypes.any
}





