import PropTypes from 'prop-types';
import {Dropdown} from "primereact/dropdown";
import {Button} from "primereact/button";

import UIConstants from "../../../utils/ui.constants";

function projectSelector(selectedProject, projectDisabled, allProjects, projectErrors, touchedProject, changeProject) {
    const dropdown = <Dropdown inputId="project"
                               optionLabel="name"
                               optionValue="name"
                               data-testid="project"
                               testid="project"
                               tooltip="Project"
                               tooltipOptions={UIConstants.tooltipOptions}
                               className="projectDropdown"
                               value={selectedProject}
                               disabled={projectDisabled}
                               options={allProjects}
                               onChange={(e) => changeProject(e.value)}
                               placeholder="Select Project"/>;
    return <div className="col-lg-3 col-md-3 col-sm-12" data-testid="projectdiv">
        {dropdown}
        <label
            className={(projectErrors && touchedProject) ? "error" : "info"}>
            {(projectErrors && touchedProject) ? projectErrors : "Select Project to get Scheduling Sets"}
        </label>
    </div>
}

function schedulingSetSelector(selectedSchedulingSet, allSchedulingSetS, schedulingSetErrors, touchedSchedulingSet, setSchedUnitParams) {
    const dropdown = <Dropdown data-testid="schedSet"
                               id="schedSet"
                               optionLabel="name"
                               optionValue="id"
                               tooltip="Scheduling set of the project"
                               tooltipOptions={UIConstants.tooltipOptions}
                               value={selectedSchedulingSet}
                               options={allSchedulingSetS}
                               onChange={(e) => setSchedUnitParams('scheduling_set_id', e.value)}
                               placeholder="Select Scheduling Set"/>;
    return <div className="col-lg-3 col-md-3 col-sm-10">
        {dropdown}
        <label
            className={(schedulingSetErrors && touchedSchedulingSet) ? "error" : "info"}>
            {(schedulingSetErrors && touchedSchedulingSet) ? schedulingSetErrors : "Scheduling Set of the Project"}
        </label>
    </div>
}

function addSchedulingSetButton(selectedProject, scheduleUnitDraft, setState) {
    return <div className="col-lg-1 col-md-1 col-sm-2">
        <Button label=""
                className="p-button-primary"
                icon="pi pi-plus"
                data-testid="add-schedulingset-button"
                onClick={() => setState({showAddSet: true})}
                tooltip="Add new Scheduling Set"
                style={{marginLeft: '-10px'}}
                disabled={!(selectedProject !== null && scheduleUnitDraft?.scheduling_set !== undefined)}/>
    </div>
}

/**
 * @prop selectedProject Object{name<string>, value<string>}
 * @prop projectDisabled boolean
 * @prop allProjects list<Object{name<string>, value<string>}>
 * @prop projectErrors <string>
 * @prop touchedProject boolean
 * @prop selectedSchedulingSet list<string> (multi-select)
 * @prop allSchedulingSets  list<Object{name<string>, value<string>, id<integer>}>
 * @prop schedulingSetErrors <string>
 * @prop touchedSchedulingSet boolean
 * @prop scheduleUnitDraft ???
 * @prop changeProject function: callback
 * @prop setSchedUnitParams function: callback
 * @prop setState function: callback
 * @return {JSX.Element}
 */
export default function ProjectScheduleSetSelector(props) {
    const {
        selectedProject,
        projectDisabled,
        allProjects,
        projectErrors,
        touchedProject,
        selectedSchedulingSet,
        allSchedulingSets,
        schedulingSetErrors,
        touchedSchedulingSet,
        scheduleUnitDraft,
        changeProject,
        setSchedUnitParams,
        setState,
    } = props

    return <div className="p-field p-grid">
        <label htmlFor="project" className="col-lg-2 col-md-2 col-sm-12">
            Project <span style={{color: 'red'}}>*</span>
        </label>
        {projectSelector(selectedProject, projectDisabled, allProjects, projectErrors, touchedProject, changeProject)}
        <div className="col-lg-1 col-md-1 col-sm-12"></div>
        <label htmlFor="schedSet" className="col-lg-2 col-md-2 col-sm-12">
            Scheduling Set <span style={{color: 'red'}}>*</span>
        </label>
        {schedulingSetSelector(selectedSchedulingSet, allSchedulingSets, schedulingSetErrors, touchedSchedulingSet, setSchedUnitParams)}
        {addSchedulingSetButton(selectedProject, scheduleUnitDraft, setState)}
    </div>
}

ProjectScheduleSetSelector.propTypes = {
  selectedProject: PropTypes.any,
  projectDisabled: PropTypes.any,
  allProjects: PropTypes.any,
  projectErrors: PropTypes.any,
  touchedProject: PropTypes.any,
  selectedSchedulingSet: PropTypes.any,
  allSchedulingSets: PropTypes.any,
  schedulingSetErrors: PropTypes.any,
  touchedSchedulingSet: PropTypes.any,
  scheduleUnitDraft: PropTypes.any,
  changeProject: PropTypes.any,
  setSchedUnitParams: PropTypes.any,
  setState: PropTypes.any
}