import {render} from '@testing-library/react';
import {clickItem, removeReact18ConsoleErrors} from "../../../utils/test.helper";
import ProjectScheduleSetSelector from "./ProjectScheduleSetSelector";

removeReact18ConsoleErrors();


describe('ProjectScheduleSetSelector', () => {
    const allProjects = [
        {name: 'Batman', value: '1'},
        {name: 'Robin', value: '2'},
    ]
    const allSchedulingSets = [
        {name: 'X-men', value: '3', id: 12},
        {name: 'Batman', value: '4', id: 13}];
    const projectErrorMsg = 'robin is not better than batman';
    const scheduleSetErrorMsg = 'batman is not part of x-men';
    const scheduleUnitDraft = {scheduling_set: allSchedulingSets[0]};

    test('Renders all elements correctly with errors', () => {
        const {getByTestId, getByText} = render(
            <ProjectScheduleSetSelector
                selectedProject={allProjects[0]}
                projectDisabled={false}
                allProjects={allProjects}
                projectErrors={projectErrorMsg}
                touchedProject={true}
                changeProject={jest.fn()}
                selectedSchedulingSet={allSchedulingSets[0]}
                allSchedulingSets={allSchedulingSets}
                schedulingSetErrors={scheduleSetErrorMsg}
                touchedSchedulingSet={true}
                setSchedUnitParams={jest.fn()}
                scheduleUnitDraft={scheduleUnitDraft}
                setState={jest.fn()}
            />
        );

        expect(getByTestId('schedSet')).toBeInTheDocument();
        expect(getByTestId('project')).toBeInTheDocument();
        expect(getByTestId('add-schedulingset-button')).toBeInTheDocument();
        expect(getByText(projectErrorMsg)).toBeInTheDocument();
        expect(getByText(scheduleSetErrorMsg)).toBeInTheDocument();
    });

    test('Renders all elements correctly without errors but disabled button', () => {
        const {getByTestId, getByText} = render(
            <ProjectScheduleSetSelector
                selectedProject={allProjects[0]}
                projectDisabled={false}
                allProjects={allProjects}
                projectErrors={undefined}
                touchedProject={false}
                changeProject={jest.fn()}
                selectedSchedulingSet={allSchedulingSets[0]}
                allSchedulingSets={allSchedulingSets}
                schedulingSetErrors={undefined}
                touchedSchedulingSet={false}
                setSchedUnitParams={jest.fn()}
                scheduleUnitDraft={undefined}
                setState={jest.fn()}
            />
        );

        expect(getByTestId('schedSet')).toBeInTheDocument();
        expect(getByTestId('project')).toBeInTheDocument();
        const buttonElement = getByTestId('add-schedulingset-button');
        expect(buttonElement).toBeInTheDocument();
        expect(buttonElement).toHaveAttribute('disabled')
        expect(() => getByText(projectErrorMsg)).toThrow();
        expect(() => getByText(scheduleSetErrorMsg)).toThrow();
    });

    test('Calls changeProject when project is changed', async () => {
        const changeProject = jest.fn();
        const content = render(
            <ProjectScheduleSetSelector
                selectedProject={allProjects[0]}
                projectDisabled={false}
                allProjects={allProjects}
                projectErrors={projectErrorMsg}
                touchedProject={true}
                changeProject={changeProject}
            />
        );

        const newProject = allProjects[1];

        await clickItem(content.getByTestId('project'))
        await clickItem(content.getByText(newProject.name))
        expect(changeProject).toHaveBeenCalledWith(newProject.name);
    });

    test('Calls setSchedUnitParams when scheduling set changed', async () => {
        const setSchedUnitParams = jest.fn();
        const content = render(
            <ProjectScheduleSetSelector
                selectedSchedulingSet={allSchedulingSets[0]}
                allSchedulingSets={allSchedulingSets}
                schedulingSetErrors={scheduleSetErrorMsg}
                touchedSchedulingSet={true}
                setSchedUnitParams={setSchedUnitParams}
            />
        );

        const newSchedulingSet = allSchedulingSets[1];

        await clickItem(content.getByTestId('schedSet'))
        await clickItem(content.getByText(newSchedulingSet.name))
        expect(setSchedUnitParams).toHaveBeenCalledWith('scheduling_set_id', newSchedulingSet.id);
    });
    test('Calls setState for modal when add scheduling set button is clicked', async () => {
        const setState = jest.fn();
        const content = render(
            <ProjectScheduleSetSelector
                selectedProject={allProjects[0]}
                scheduleUnitDraft={scheduleUnitDraft}
                setState={setState}
            />
        );

        await clickItem(content.getByTestId('add-schedulingset-button'))
        expect(setState).toHaveBeenCalledWith({showAddSet: true});
    });

});