import { useRef, useState } from 'react';
import PropTypes from 'prop-types';
import { Button } from 'primereact/button';
import { ConfirmPopup  } from 'primereact/confirmpopup';
import { Calendar  } from 'primereact/calendar';
import StationConstraintImage from './StationConstraintImage'
import ScheduleService from "../../services/schedule.service";
import { Toast } from 'primereact/toast';
export default function StationConstraintView(props) {
    const {
        station,
        selectedBluePrint,
        selectedBluePrintId,
        startTime
    } = props
    const toast = useRef(null);
    const [visible, setVisible] = useState(false);
    const [scheduletime, setScheduletime] = useState(new Date().now);
    const buttonSchedule = useRef(null);


    const accept = () => {
        let CombinedTime = startTime+"T"+scheduletime.getHours()+":"+ scheduletime.getMinutes()+":00"
        ScheduleService.ScheduleBluePrintOn(selectedBluePrintId,CombinedTime);
        toast.current.show({ severity: 'info', summary: 'Confirmed', detail: 'Setting Schedule for ' + selectedBluePrint[1].name + ' at ' + startTime, life: 3000 });
    };

    return <div className='ConstraintsStart'>
        {selectedBluePrint ? (
            <StationConstraintImage blueprintId={selectedBluePrintId} station={station} startTime={startTime} >

                { selectedBluePrint[1].status==="scheduled" ||selectedBluePrint[1].status==="schedulable" ? (
                    <>
                <ConfirmPopup target={buttonSchedule.current} visible={visible} onHide={() => setVisible(false)}  message="Are you sure you want set the blueprint to a fixed time ?"  icon="pi pi-exclamation-triangle" accept={accept}  />
                <div className='HeaderBlock'> 
                <span className="scheduletime"> Schedule time <Calendar value={scheduletime} onChange={(e) => setScheduletime(e.value)} timeOnly /> </span>
                <Button title="select contraint" ref={buttonSchedule} label={"Schedule "  + selectedBluePrint[1].name  + " at "+ startTime} className='autowidthmargin' onClick={() => setVisible(true)} disabled={!scheduletime}></Button>
                
                </div>
                <Toast ref={toast} />
                </>
                ):(<></>)}
            </StationConstraintImage>)
            : (<div></div>
            )}
    </div>

}

StationConstraintView.propTypes = {
  station: PropTypes.string,
  selectedBluePrint: PropTypes.array,
  selectedBluePrintId: PropTypes.string,
  startTime: PropTypes.string
}