import PropTypes from 'prop-types';
import { Chips } from "primereact/chips"
import { Dropdown } from "primereact/dropdown"
import { InputText } from "primereact/inputtext"
import { InputTextarea } from "primereact/inputtextarea"
import { Link } from "react-router-dom"

const TaskEditHeader = (props) => {

    const errors = props.errors
    const task = props.task
    const setTaskParams = props.setTaskParams
    const schedulingUnit = props.schedulingUnit
    const taskTemplates = props.taskTemplates
    const changeTaskTemplate = props.changeTaskTemplate
    
return <div>
<div className="p-fluid">
    <div className="p-field p-grid">
        <label htmlFor="taskName" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{ color: 'red' }}>*</span></label>
        <div className="col-lg-4 col-md-4 col-sm-12">
            <InputText className={errors.name ? 'input-error' : ''} id="taskName" type="text" value={task.name} disabled={true}
                data-testid="taskName" onChange={(e) => setTaskParams('name', e.target.value)} />
            <label className="error">
                {errors.name ? errors.name : ""}
            </label>
        </div>
        <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{ color: 'red' }}>*</span></label>
        <div className="col-lg-4 col-md-4 col-sm-12">
            <InputTextarea className={errors.description ? 'input-error' : ''} rows={3} cols={30}
                data-testid="taskDescription" value={task.description} onChange={(e) => setTaskParams('description', e.target.value)} />
            <label className="error">
                {errors.description ? errors.description : ""}
            </label>
        </div>
    </div>
    <div className="p-field p-grid">
        <label htmlFor="tags" className="col-lg-2 col-md-2 col-sm-12">Tags</label>
        <div className="col-lg-4 col-md-4 col-sm-12">
            <Chips value={task.tags ? task.tags : []} onChange={(e) => setTaskParams('tags', e.value)}></Chips>
        </div>
        {schedulingUnit &&
            <>
                <label className="col-lg-2 col-md-2 col-sm-12">Scheduling Unit</label>
                <Link className="col-lg-4 col-md-4 col-sm-12" to={{ pathname: `/schedulingunit/view/draft/${schedulingUnit.id}`, state: { id: schedulingUnit.id } }}>{schedulingUnit ? schedulingUnit.name : ''}</Link>
            </>
        }
    </div>
    <div className="p-field p-grid">
        <label htmlFor="tags" className="col-lg-2 col-md-2 col-sm-12">Template</label>
        <div className="col-lg-4 col-md-4 col-sm-12">
            <Dropdown optionLabel="name" optionValue="id"
                value={task.specifications_template_id}
                options={taskTemplates} disabled
                onChange={(e) => { changeTaskTemplate(e.value) }}
                placeholder="Select Task Template" />
        </div>
        <label htmlFor="short_description" className="col-lg-2 col-md-2 col-sm-12">Short Description</label>
        <div className="col-lg-4 col-md-4 col-sm-12">
            <InputText className={errors.short_description ? 'input-error' : ''} id="taskShortDescription" maxLength={32} type="text" value={task.short_description}
                data-testid="taskShortDescription" onChange={(e) => setTaskParams('short_description', e.target.value)} />
            <label className="info">Max 32 characters</label>
        </div>
    </div>
</div>
</div>
}

TaskEditHeader.propTypes = {
  errors: PropTypes.shape({
    name: PropTypes.any,
    description: PropTypes.any,
    short_description: PropTypes.any
  }),
  task: PropTypes.shape({
    name: PropTypes.any,
    description: PropTypes.any,
    tags: PropTypes.any,
    specifications_template_id: PropTypes.any,
    short_description: PropTypes.any
  }),
  setTaskParams: PropTypes.func,
  schedulingUnit: PropTypes.shape({
    id: PropTypes.any,
    name: PropTypes.any
  }),
  taskTemplates: PropTypes.any,
  changeTaskTemplate: PropTypes.func
}

export default TaskEditHeader