import PropTypes from 'prop-types';
import UIConstants from "../../../utils/ui.constants";

function getShowOrHideButton(title, show, setShow, icon) {
    return <div className="resize-div su-hidden">
        <button className={"p-link resize-btn " + (show ? "active" : "")}
                title={title}
                data-testid={title + "-button"}
                onClick={() => setShow(!show)}>
            <i className={"pi " + icon}> {title}</i>
        </button>
    </div>
}

function calculateLeftPanelSize(currentLeftPanelSize, amount, minSize, maxSize) {
    const newResizeAmount = currentLeftPanelSize + amount
    if (newResizeAmount > maxSize) {
        return maxSize
    } else if (newResizeAmount < minSize) {
        return minSize
    }
    return newResizeAmount
}

function getResizeButtons(showButtons, showSummary, currentLeftPanelSize, setLeftPanelSize) {
    if (!showButtons===true) {
        return null;
    }

    const minSize = UIConstants.PANEL_SIZE_CLASS_MAP[1].size;
    const maxSize = UIConstants.PANEL_SIZE_CLASS_MAP[UIConstants.PANEL_SIZE_CLASS_MAP.length - 1].size
    return <div className="resize-div small-buttons">
        <button className="p-link resize-btn"
                disabled={currentLeftPanelSize === minSize || showSummary} //when summary is active, do not resize
                title="Shrink List/Expand Timeline"
            //decrease with 10%
                onClick={() => setLeftPanelSize(calculateLeftPanelSize(currentLeftPanelSize, -10, minSize, maxSize))}>
            <i className="pi pi-step-backward"></i>
        </button>
        <button className="p-link resize-btn"
                disabled={currentLeftPanelSize === maxSize || showSummary}  //when summary is active, do not resize
                title="Expand List/Shrink Timeline"
            //increase with 10%
                onClick={() => setLeftPanelSize(calculateLeftPanelSize(currentLeftPanelSize, 10, minSize, maxSize))}>
            <i className="pi pi-step-forward"></i>
        </button>
    </div>
}

export default function PanelButtons(props) {

    const {
        showLegendbar,
        setShowLegendbar,
        showListTabs,
        setShowListTabs,
        showSummary,
        leftPanelSize,
        setLeftPanelSize
    } = props

    const legendTitle = showLegendbar ? "Hide Legend" : "Show Legend"
    const legendIcon = showLegendbar ? "pi-filter-slash" : "pi-filter"
    const listTabTitle = showListTabs ? "Hide List" : "Show List"
    const listTabIcon = showListTabs ? "pi-eye-slash" : "pi-eye"

    const showResizeButtons = showLegendbar || showListTabs

    return <div className={"panel-buttons-container" + (showResizeButtons ? "" : " show-buttons")}
                id="parent-panel-buttons">
        {getResizeButtons(showResizeButtons, showSummary, leftPanelSize, setLeftPanelSize)}
        {showLegendbar !== undefined && getShowOrHideButton(legendTitle, showLegendbar, setShowLegendbar, legendIcon)}
        {showListTabs !== undefined && getShowOrHideButton(listTabTitle, showListTabs, setShowListTabs, listTabIcon)}
    </div>

}

PanelButtons.propTypes = {
  showLegendbar: PropTypes.any,
  setShowLegendbar: PropTypes.any,
  showListTabs: PropTypes.any,
  setShowListTabs: PropTypes.any,
  showSummary: PropTypes.any,
  leftPanelSize: PropTypes.any,
  setLeftPanelSize: PropTypes.any
}

