import PropTypes from 'prop-types';
import Flatpickr from "react-flatpickr";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import { useEffect, useState } from "react";
import JumpToSUId from "./JumpToSuid";
import moment from "moment";
import UIConstants from "../../../../utils/ui.constants";

import axios from "axios";
import { getStore } from "../../../../services/store.helper";
import { Panel } from 'primereact/panel';


function DateSelector(props) {  //TODO: extract to separate class
    const {
        startTime,
        setStartAndEndTimeCallback,
    } = props

    const DayorWeekOptions = ["Showing Day At", "Showing Week From"];
    const [selectedDayWeekOptions, setSelectedDayWeekOptions] = useState("Showing Week From")

    const options = {
        "allowInput": true,
        "mode": "single",
        "minuteIncrement": 1,
        "defaultHour": 0,
        "wrap": true,
        "inlineHideInput": true,
    }

    useEffect(() => {
        UpdateTime(startTime)
    }, [selectedDayWeekOptions])


    const UpdateTime = (startTime) => {
        const currentStartTime = moment(startTime)
        let numberDays = 6
        if (selectedDayWeekOptions == DayorWeekOptions[0]) {
            numberDays = 0
        }
        const newEndTime = currentStartTime.clone().add(numberDays, 'days').endOf('day')
        setStartAndEndTimeCallback(currentStartTime, newEndTime)
    }

    return <div data-testid="nav-datetime-selector">

        <Dropdown value={selectedDayWeekOptions} onChange={(e) => { setSelectedDayWeekOptions(e.value) }} options={DayorWeekOptions} className="DayWeekDropDown" />
        <Flatpickr data-enable-time={false}
            data-input
            value={startTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)}
            options={options}
            onClose={value => {
                const chosenDate = moment(value[0]);
                const newStartTime = moment(chosenDate.startOf('day'))
                UpdateTime(newStartTime)
            }}>
            <input type="text" data-input className="p-inputtext p-component calendar-input" />
            <Button className="p-button-icon-only calendar-button"
                data-testid="nav-reset-button"
                onClick={() => {
                    const now = moment(moment.now())
                    const newStartTime = moment(now.startOf('day'));
                    UpdateTime(newStartTime)
                }}
                title="Reset to default">
                <i className="fas fa-undo"></i>
            </Button>
        </Flatpickr>
    </div>
}

DateSelector.propTypes = {
  startTime: PropTypes.shape({
    format: PropTypes.func
  }),
  setStartAndEndTimeCallback: PropTypes.func
}




function changeStartAndEndTimeCallback(setStartTime, setEndTime) {
    return (newStartTime, newEndTime) => {
        setStartTime(newStartTime);
        setEndTime(newEndTime);
    };
}

export default function DateTimeNavigator(props) {
    const {
        startTime,
        setStartTime,
        setEndTime
    } = props


    const [suIdInput, setSUIdInput] = useState()
    const [searchForSUIdTrigger, setSearchForSUIdTrigger] = useState(false)

    useEffect(() => {
        async function fetchStartTimeOfSUId(suId) {
            const shouldUseOnSkyTimes = getStore(UIConstants.STORE_KEY_TIMELINE).onSkyToggle ?? true
            const timeField = shouldUseOnSkyTimes ? "on_sky_start_time" : "process_start_time"
            const response = await axios.get('/api/scheduling_unit_blueprint/' + suId + "?fields=" + timeField);
            const newStartTime = moment(response.data[timeField]).startOf('day')
            const newEndTime = moment(moment(newStartTime).add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days').endOf('day'))
            if (newStartTime.isValid() && newEndTime.isValid()) {
                setStartTime(newStartTime);
                setEndTime(newEndTime);
            } else {
                console.error("Invalid start/end times found when jumping to the SU id's week.")
            }
        }

        if (suIdInput) {
            fetchStartTimeOfSUId(suIdInput).catch(e => console.error("Couldn't fetch the start time for SU id:", suIdInput, e))
        }
    }, [searchForSUIdTrigger])

    return <div className="p-grid timeline-datetime-navigator">
        <Panel header="Navigation">
            <div className="group">

                <div className="selector-container">
                    <DateSelector startTime={startTime}
                        setStartAndEndTimeCallback={changeStartAndEndTimeCallback(setStartTime, setEndTime)} />
                    <JumpToSUId suId={suIdInput} setSUId={setSUIdInput}
                        setSearchForSUIdTrigger={setSearchForSUIdTrigger} />
                </div>
            </div>
        </Panel>
    </div>
}

DateTimeNavigator.propTypes = {
  startTime: PropTypes.any,
  setStartTime: PropTypes.func,
  setEndTime: PropTypes.func
}