import moment from "moment/moment";

import UtilService from "../../../services/util.service";
import { getTimelineGroups } from "../business/timelinegroupingbusiness.js";
import UIConstants from "../../../utils/ui.constants.js";
import PageHeaderActions from "../../../utils/pageheaderactions.util.js";
import { fetchLSTStationShift } from "../data/week.view.data.js"



export async function updateSchedulerStatuses(switchValue, status, setSchedulerSettings) {
    if (status === "fixed") {
        setSchedulerSettings(prevState => ({
            ...prevState,
            isFixedTimeSchedulerOn: switchValue
        }))
    }
    if (status === "dynamic") {
        let isDynamicSchedulerActive = false
        if (switchValue) {
            await UtilService.getSubSystemScheduler()
                .then(response => isDynamicSchedulerActive = response ? response.status_value === "active" : false);
        }
        setSchedulerSettings(prevState => ({
            ...prevState,
            isDynamicSchedulerOn: switchValue,
            isDynamicSchedulerActive: isDynamicSchedulerActive
        }))
    }
}



export function getHeaderActions(isLoading, setRefetchToggle, actionsMenuRef, isExpanded, setIsExpanded, isLegendVisible, setIsLegendVisable, isTableViewVisible, setIsTableViewVisible) {
    return [
        PageHeaderActions.actionButtonObject(
            isLoading ? "System is already reloading the timeline data" : "Reload the timeline data",
            (isLoading ? "pi-spin pi-hourglass " : "pi-sync"),
            `subsystem subsystem--${isLoading ? "on" : "standard"}`,
            { callback: () => setRefetchToggle(prevState => !prevState) }
        ),
        PageHeaderActions.actionButtonObject(
            "Options",
            "pi-bars",
            "",
            { callback: (ev) => actionsMenuRef.current.toggle(ev) }
        ),

        

        PageHeaderActions.actionButtonObject(
            (isTableViewVisible ? "Hide Scheduling Units Table View" : "Show Scheduling Units Table"),
            (isTableViewVisible ? "pi-folder-open" : "pi-folder"),
            (isTableViewVisible ? "" : "color:grey"),
            { callback: () => setIsTableViewVisible(prevState => !prevState) }
        ),

        PageHeaderActions.actionButtonObject(
            (isLegendVisible ? "Hide Legend" : "Show Legend"),
            (isLegendVisible ? "pi-bookmark" : "pi-bookmark-fill"),
            (isLegendVisible ? "" : "color:grey"),
            { callback: () => setIsLegendVisable(prevState => !prevState) }
        ),

        PageHeaderActions.actionButtonObject(
            (isExpanded ? "Hide Filters" : "Show Filter"),
            (isExpanded ? "pi-chevron-up" : "pi-chevron-down"),
            "",
            { callback: () => setIsExpanded(prevState => !prevState) }
        ),


    ];
}
/**
   * getGroupedByActions
   * Returns actions used to group by
   * @param {function}setGroupedBy {
   */
export function getGroupedByActions(selected, setGroupedBy) {
    return [
        PageHeaderActions.actionDropDownObject(
            "Group By",
            getTimelineGroups(),
            { callback: (groupingby) => setGroupedBy(groupingby) },
            selected,
            "timeline-header-dropdown", "value", "label"
        ),
    ]

}







export function getSchedulerActions(schedulerSettings, setShowSchedulerSettingsDialog) {
    return [
        PageHeaderActions.actionTagObject(
            schedulerSettings.isDynamicSchedulerOn ? "Dynamic Scheduling is On" : "Dynamic Scheduling is Off",
            "D",
            "tag tag--" + (schedulerSettings.isDynamicSchedulerOn ? "on" : "off")
        ),
        PageHeaderActions.actionTagObject(
            schedulerSettings.isFixedTimeSchedulerOn ? "Fixed time Scheduling is On" : "Fixed time Scheduling is Off",
            "F",
            "tag tag--" + (schedulerSettings.isFixedTimeSchedulerOn ? "on" : "off")
        ),
        PageHeaderActions.actionButtonObject(
            (schedulerSettings.isDynamicSchedulerActive ? "Dynamic Scheduler is Active" :
                (schedulerSettings.isDynamicSchedulerOn ? "Dynamic Scheduler is Idle" : "Dynamic Scheduler is stopped")) + ". Click to change the scheduler",
            "pi-" + (schedulerSettings.isDynamicSchedulerActive ? "play" : (schedulerSettings.isDynamicSchedulerOn ? "pause" : "stop")),
            "subsystem subsystem--" + (schedulerSettings.isDynamicSchedulerActive ? "on" : (schedulerSettings.isDynamicSchedulerOn ? "paused" : "off")),
            { callback: () => setShowSchedulerSettingsDialog(true) }
        ),



    ];
}

export function getPageHeaderOptionsMenuItems(permissions,fromdate,todate) { //TODO factory method
    return [
        {
            label: 'Filtered Reservations',
            icon: "pi pi-book",
            disabled: permissions == null || permissions.weekView == null || !permissions.weekView.listreservation,
            command: () => window.open("/reservation/list/from/" + fromdate.format("YYYY-MM-DDTHH:mm:ss") + "/to/"+ todate.format("YYYY-MM-DDTHH:mm:ss"), '_blank')
        },
        {
            label: 'Add Reservation',
            icon: "pi pi-plus",
            disabled: permissions == null || permissions.weekView == null || !permissions.weekView.addreservation,
            command: () => window.open('/reservation/create', '_blank')
        },
        
        {
            separator: true
        },

        {
            label: 'Filtered System Events',
            icon: "pi  pi-bolt",
            disabled: permissions == null || permissions.weekView == null || !permissions.weekView.listreservation,
            command: () => window.open("/systemevent/list/from/" + fromdate.format("YYYY-MM-DDTHH:mm:ss") + "/to/"+ todate.format("YYYY-MM-DDTHH:mm:ss"), '_blank')
        },
        {
            label: 'Add System Event',
            icon: "pi pi-plus",
            disabled: permissions == null || permissions.weekView == null || !permissions.weekView.addsystemevent,
            command: () => window.open('/systemevent/create', '_blank')

        },
        {
            separator: true
        },
        {
            label: 'Add Scheduling Unit',
            icon: "pi pi-plus",
            disabled: permissions == null || permissions.schedulingUnit == null || !permissions.schedulingUnit.create,
            command: () => window.open('/schedulingunit/create', '_blank')

        },
    ];
}

const timecache = {}; // cache per day per station

export async function getCursorDateTimes(cursorTimeUTC, groupIndex, groupDate, lstShiftInSeconds, stationName, isSingleStation, setCursorDateTimes) {
    const cursorTime = moment(cursorTimeUTC)
    const cursorDateTimeUTC = moment(groupDate).set({ hour: cursorTime.get('hour'), minute: cursorTime.get('minute'), second: cursorTime.get('second') });
    let utc = cursorDateTimeUTC.format("HH:mm");
    let shifts = await GetStationLst(isSingleStation, cursorDateTimeUTC, stationName);
    let coreLST = shifts.coreLST;
    let stationLST = shifts.stationLST;
    let cursordatetimes = { utc, coreLST, stationLST, stationName: stationName }
    setCursorDateTimes(cursordatetimes)
}


async function GetStationLst(isSingleStation, cursorDateTimeUTC, stationName = UIConstants.TIMINGSERVER) {

    let lststationshiftSeconds = -2;
    let shift;
    let key = cursorDateTimeUTC.format(UIConstants.CALENDAR_GROUPING_FORMAT) + "_" + stationName; // we cache on a daily base
    if (key in timecache) {
        if (!timecache[key].isRetrieved) { return "..."; }
        shift = timecache[key].shift;
        lststationshiftSeconds = shift.shiftLSTStation;
    }
    if (lststationshiftSeconds == -2) {
        timecache[key] = { isRetrieved: false };
        shift = await fetchLSTStationShift(cursorDateTimeUTC, stationName);
        timecache[key] = { isRetrieved: true, shift };
    }
    const stationLST = moment(cursorDateTimeUTC).add(shift.shiftLSTStation, 'second').format("HH:mm");
    const coreLST = moment(cursorDateTimeUTC).add(shift.shiftLSTCore, 'second').format("HH:mm");



    return { stationLST, coreLST };

}

function getSunTimingItem(groupId, groupkey, idPrefix, color, startTime, sunTimeStart, sunTimeEnd, fullOpacity = false) {

    const groupitem =
    {
        id: `${idPrefix}-${groupId}`,
        title: "",
        start_time: startTime.clone().hours(sunTimeStart.hours()).minutes(sunTimeStart.minutes()).seconds(sunTimeStart.seconds()),
        end_time: startTime.clone().hours(sunTimeEnd.hours()).minutes(sunTimeEnd.minutes()).seconds(sunTimeEnd.seconds()),
        bgColor: color,
        selectedBgColor: color,
        opacity: fullOpacity ? "0%" : "50%",
        type: "SUNTIME"
    };
    groupitem[groupkey] = groupId;
    return groupitem;
}


export async function getSunTimingItems(startTime, groups, groupkey) {

    let uniqueStations = [...new Set(groups.map(group => group.designatedStation))]; // get all unique Station names.
    let stringifiedUniqueStations = uniqueStations.join(",");
    let uniqueDates = [...new Set(groups.map(group => moment(group.date).format("YYYY-MM-DD")))]; // get all unique Station dates.
    let stringifiedUniqueDates = uniqueDates.join(",");
    const allstationtimings = await UtilService.getSunTimings(stringifiedUniqueDates, stringifiedUniqueStations);

    const sunTimingItems = []
    for (const group of groups) {
        const groupDay = moment(group.date);
        const groupDaymoment = groupDay.format("YYYY-MM-DD");
        let timings = allstationtimings?.[group.designatedStation || "CS002"];
        let dateindex = uniqueDates.indexOf(groupDaymoment);

        if (timings?.sunrise?.length > dateindex) {
            const sunriseStart = moment.utc(timings.sunrise[dateindex].start);
            const sunriseEnd = moment.utc(timings.sunrise[dateindex].end);
            const sunsetStart = moment.utc(timings.sunset[dateindex].start);
            const sunsetEnd = moment.utc(timings.sunset[dateindex].end);
            const startOfDay = startTime.clone().startOf('day')
            const endOfDay = startTime.clone().endOf('day')
            const nightItemBeforeSunrise = getSunTimingItem(group.id, groupkey, "bef-sunrise", "grey", startOfDay, startOfDay, sunriseStart)
            const sunriseItem = getSunTimingItem(group.id, groupkey, "sunrise", "#FFF176", startTime, sunriseStart, sunriseEnd)
            const sunsetItem = getSunTimingItem(group.id, groupkey, "sunset", "#FFB74D", startTime, sunsetStart, sunsetEnd)
            const dayItem = getSunTimingItem(group.id, groupkey, "day", "white", startTime, sunriseEnd, sunsetStart, true)
            const nightItemAfterSunset = getSunTimingItem(group.id, groupkey, "aft-sunrise", "grey", startTime, sunsetEnd, endOfDay)
            sunTimingItems.push(nightItemBeforeSunrise, sunriseItem, dayItem, sunsetItem, nightItemAfterSunset)
        }
    }

    return sunTimingItems;

}