import PropTypes from 'prop-types';
import  { Component } from 'react';
import { Button } from 'primereact/button';
import SunEditor from 'suneditor-react';
import 'suneditor/dist/css/suneditor.min.css'; // Import Sun Editor's CSS File
import { Checkbox } from 'primereact/checkbox';
import WorkflowService from '../../services/workflow.service';
import { Link } from 'react-router-dom';
import TaskAssigner from './task.assigner';
import { SystemEventList } from '../SystemEvent/system.event.list';

class QAreportingSDCO extends Component {
    constructor(props) {
        super(props);
        this.state = {
            content: '',
            showEditor: false,
            quality_within_policy: true,
            sos_accept_show_pi: true,
            operator_accept: true,
            currentWorkflowTask:{},
        };
        this.user =JSON.parse(localStorage.getItem("user"));
        this.Next = this.Next.bind(this);
        this.handleChange = this.handleChange.bind(this);
        this.getQASOSDetails = this.getQASOSDetails.bind(this);
        this.assignTaskCB = this.assignTaskCB.bind(this);
    }

    async componentDidMount() {
        if (this.props.readOnly) {
            this.getQASOSDetails()
        }   else {
            const currentWorkflowTask = await this.props.getCurrentTaskDetails();
            const response = await WorkflowService.getQAReportingTo(this.props.process.qa_reporting_to);
            const processPermission = await this.props.processPermission();
            this.setState({
                content: response.operator_report,
                operator_accept: response.operator_accept, processPermission: processPermission, 
                assignTo: currentWorkflowTask.owner_id, currentWorkflowTask: currentWorkflowTask
            });
        }
    }

    async getQASOSDetails() {
        
        
        let QASOSresponse;
        if (this.props.process.qa_reporting_sos !== null && this.props.process.operator_report !== null) {
            QASOSresponse = await WorkflowService.getQAReportingSOS(this.props.process.qa_reporting_sos, this.props.process.operator_report);
        }



        this.setState({
            assignTo: this.props.workflowTask?this.props.workflowTask?.owner:null,
            content: QASOSresponse?.sos_report,
            quality_within_policy: QASOSresponse?.quality_within_policy,
            sos_accept_show_pi: QASOSresponse?.sos_accept_show_pi            
        });
    }

    /**
     * Method will trigger on change of sun-editor
     */
    handleChange(e) {
        if (e === '<p><br></p>') {
            this.setState({ content: '' });
            return;
        }
        this.setState({ content: e });
    }

     /**
     * Method will trigger on click Next buton
     * here onNext props coming from parent, where will handle redirection to other page
     */
    async Next() {
        let currentWorkflowTask = await this.props.getCurrentTaskDetails();
        const promise = [];
        promise.push(WorkflowService.updateQA_Perform(this.props.id, {"sos_report": this.state.content, "sos_accept_show_pi": this.state.sos_accept_show_pi, "quality_within_policy": this.state.quality_within_policy}));
        Promise.all(promise).then(async(responses) => {
            if (responses.indexOf(null)<0) {
                // After completing the current task, get the next task triggered in the workflow and assign project role user for the next task.
                currentWorkflowTask = await this.props.getCurrentTaskDetails();
                if (currentWorkflowTask && !currentWorkflowTask.owner_id) {
                    await WorkflowService.updateAssignTo(currentWorkflowTask.id, "project_role=contact", { owner: this.state.assignTo });
                }
                this.props.onNext({ report: this.state.content });
            }   else {
                this.props.onError('error', 'Unable to proceed', 'Please clear your browser cookies and try again');
            }
        });
    }
  
    //Not using at present
    cancelCreate() {
        this.props.history.goBack();
    }

    /**
     * Callback function passed to the Task Assigner and is called after assigning or reassigning task owner.
     */
    async assignTaskCB() {
        let currentWorkflowTask = await this.props.getCurrentTaskDetails();
        this.setState({currentWorkflowTask: currentWorkflowTask, 
                        assignTo: currentWorkflowTask.owner_id, reassign: false});
    }

    render() {
        return (
            <>
                <div>                    
                    <div className={`p-fluid-grid workflow-wrap`}>
                        <div className="p-field p-grid workflow-wrap" style={{ paddingLeft: '-10px' }}>
                            <label htmlFor="assignTo" className="col-lg-2 col-md-2 col-sm-12">Assign To</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="assignTo" >
                            {/* Display the assigned owner of the task */}
                            {this.state.assignTo &&
                                <>{!this.props.readOnly ? this.state.currentWorkflowTask.owner_username: this.props.workflowTask.owner_username }
                                    {/* Allow to edit the owner if the task is not yet completed */}
                                    {!this.props.readOnly && !this.state.reassign &&
                                <>
                                    {this.state.currentWorkflowTask.editPermissions
                                    ? <Link onClick={() =>this.setState({reassign: true})} style={{marginLeft: '10px'}}  to="#" >
                                        <i className="pi pi-pencil"></i>
                                      </Link>
                                    : <Link style={{ cursor: 'default', pointerEvents: "none", color: 'grey', marginLeft: '10px'}} onClick={() =>this.setState({reassign: true})} to="#">
                                        <i className="pi pi-pencil"></i>
                                      </Link>
                                    }
                                </>       
                            }
                                </>
                            }
                            {/* Display the task assigner if owner is not assigned or to reassign to other user */}
                            {(!this.state.assignTo || this.state.reassign) && this.state.currentWorkflowTask &&
                                <TaskAssigner currentWorkflowTask={this.state.currentWorkflowTask} disableAssignToMe = {this.state.currentWorkflowTask.owner_username === this.user.name? true: false}
                                        projectRoles={this.props.projectRoles} callback={this.assignTaskCB} />
                            }
                            </div>
                        </div>                        
                        <div className="p-grid workflow-wrap" style={{ padding: '10px' }}>
                            <label htmlFor="operatorReport">Operator Report 
                            {(!this.state.showEditor && !this.props.readOnly) && 
                                <>
                                <span className="con-edit">(Click content to edit)</span>
                                <span style={{color:'red'}}>*</span>
                                </>
                            }</label>
                            <div className="col-lg-12 col-md-12 col-sm-12"></div>
                            {this.state.showEditor && <SunEditor setDefaultStyle="min-height: 250px; height: auto" enableToolbar={true}
                                onChange={this.handleChange}
                                setContents={this.state.content}
                                setOptions={{
                                    buttonList: [
                                        ['undo', 'redo', 'bold', 'underline', 'fontColor', 'table', 'link', 'image', 'video', 'italic', 'strike', 'subscript',
                                            'superscript', 'outdent', 'indent', 'fullScreen', 'showBlocks', 'codeView', 'preview', 'print', 'removeFormat']
                                    ]
                                }}
                            />}
                            {!this.state.showEditor && <div id="operator-reportlink" onClick={() => !this.props.readOnly && this.setState({ showEditor: !this.state.showEditor })} className="operator-report" dangerouslySetInnerHTML={{ __html: this.state.content }}></div>}
                        </div>
                        <div className="p-field p-grid workflow-wrap">
                            <label htmlFor="systemissues" >System Events</label>
                        </div>
                        <div>
                            <SystemEventList schedulingUnit={this.props.schedulingUnit}/>
                        </div>
                        <div className="p-col-12" style={{marginTop: '1em', paddingLeft: '0rem'}}>
                            <Checkbox inputId="operator_accept" checked={this.state.operator_accept} disabled />
                            <label htmlFor="operator_accept" style={{paddingLeft:"5px"}}>The data quality adheres to policy (Operator evaluation)</label>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>   
                            <Checkbox inputId="quality_within_policy" disabled={this.props.readOnly} checked={this.state.quality_within_policy} onChange={e => this.setState({quality_within_policy: e.checked})} />
                            <label htmlFor="qualityPolicy"  style={{paddingLeft:"5px"}}>The data quality adheres to policy (SDCO evaluation)</label>
                            <div className="col-lg-1 col-md-1 col-sm-12"></div>      
                            <Checkbox inputId="sos_accept_show_pi" disabled={this.props.readOnly} checked={this.state.sos_accept_show_pi} onChange={e => this.setState({ sos_accept_show_pi: e.checked })} />
                            <label htmlFor="sdcoAccept" style={{paddingLeft:"5px"}}>SDCO Accepts</label>
                        </div>
                    </div>
                    {!this.props.readOnly && <div className="p-grid" style={{ marginTop: '20px' }}>
                        <div className="btn-bar">
                            <Button disabled={this.state.content && this.state.processPermission && this.state.currentWorkflowTask.editPermissions && this.state.currentWorkflowTask.owner_username === this.user.name? false: true } 
                                label="Next" className="p-button-primary" icon="pi pi-check" onClick={ this.Next } />
                        </div>
                        <div className="btn-bar">
                            <Button label="Cancel" disabled={this.props.readOnly} className="p-button-danger" icon="pi pi-times"  style={{ width : '90px' }} 
                                onClick={()=>this.props.onCancel()} />
                        </div>
                    </div>}
                    {!this.props.readOnly && 
                        <>
                            {this.state.currentWorkflowTask.editPermissions && this.state.currentWorkflowTask.owner_username === this.user.name? 
                                <span></span>: <span style={{color: 'red'}}>* You can only save and proceed next if you are assigned to this workflow step</span>
                            }
                        </>
                    }
                </div>
            </>
        )
    }
}

QAreportingSDCO.propTypes = {
  readOnly: PropTypes.any,
  getCurrentTaskDetails: PropTypes.func,
  process: PropTypes.shape({
    qa_reporting_to: PropTypes.any,
    qa_reporting_sos: PropTypes.any,
    operator_report: PropTypes.any
  }),
  processPermission: PropTypes.func,
  workflowTask: PropTypes.shape({
    owner: PropTypes.any,
    owner_username: PropTypes.any
  }),
  id: PropTypes.any,
  onNext: PropTypes.func,
  onError: PropTypes.func,
  history: PropTypes.object,
  projectRoles: PropTypes.any,
  schedulingUnit: PropTypes.any,
  onCancel: PropTypes.func
}
export default QAreportingSDCO;