import axios from 'axios'
import TaskService from './task.service';
import moment from 'moment';
import _ from 'lodash';
import {appGrowl} from '../layout/components/AppGrowl';
import ShowErrorDetails from '../show.error.details';
import UtilService from './util.service';

const SU_EXPAND_FIELDS = ["draft", "draft.observation_strategy_template", "draft.scheduling_constraints_doc",
    "draft.scheduling_set", "draft.scheduling_set.project",
    "task_blueprints", "task_blueprints.specifications_template", "task_blueprints.subtasks",
    "task_blueprints.subtasks.output_dataproducts"];

const SU_FETCH_FIELDS = ["id",
    "name",
    "description",
    "short_description",
    "status",
    "output_pinned",
    "results_accepted",
    "unschedulable_reason",
    "scheduling_constraints_doc",
    "scheduling_constraints_template_id",
    "do_cancel",
    "start_time",
    "stop_time",
    "priority_queue_value",
    "process_start_time", "process_stop_time",
    "on_sky_start_time", "on_sky_stop_time",
    "scheduled_start_time", "scheduled_stop_time",
    "duration", "on_sky_duration",
    "rank",
    "updated_at",
    "draft.id",
    "draft.name",
    "draft.observation_strategy_template.name",
    "draft.scheduling_set.project.name",
    "draft.scheduling_set.project.rank",
    "draft.scheduling_constraints_template_id",
    "draft.scheduling_constraints_doc",
    "task_blueprints.id",
    "task_blueprints.name",
    "task_blueprints.description",
    "task_blueprints.short_description",
    "task_blueprints.status",
    "task_blueprints.unschedulable_reason",
    "task_blueprints.start_time",
    "task_blueprints.stop_time",
    "task_blueprints.process_start_time", "task_blueprints.process_stop_time",
    "task_blueprints.on_sky_start_time", "task_blueprints.on_sky_stop_time",
    "task_blueprints.scheduled_start_time", "task_blueprints.scheduled_stop_time",
    "task_blueprints.task_type",
    "task_blueprints.do_cancel",
    "task_blueprints.duration",
    "task_blueprints.updated_at",
    "task_blueprints.draft_id",
    "task_blueprints.specifications_template",
    "task_blueprints.specifications_doc",
    "task_blueprints.subtasks.id",
    "task_blueprints.subtasks.output_dataproducts.size",
    "task_blueprints.subtasks.output_dataproducts.deleted_since",
    "task_blueprints.subtasks.specifications_doc",
    "task_blueprints.subtasks.subtask_type",
    "task_blueprints.obsolete_since",
    "task_blueprints.subtasks.primary"];

const ScheduleService = {


          /**
     * Function to give back a scheduling constraint url
     * @param {string} station - The Station
     * @param {string} schedulingUnit - the Unit we are looking at
     * @param {string} onDate - 2023-02-14 formatedf formated of the date
     * @returns string  with the url
     */
          getConstraintImgForStationAndSchedulingUnitOnDate : async function(station,schedulingUnit,onDate) {
           
                if (station===undefined) return;
                if (schedulingUnit===undefined) return;
                if (onDate===undefined) return;
                try {
                    let imageurl =   "/api/scheduling_unit_blueprint/" + schedulingUnit + "/scheduling_constraints_plot/" + station + "/" + onDate
                    const response = await axios.get(
                        imageurl,
                        {
                            responseType: "blob", 
                        }
                    );
                    // Convert the blob to a data URL
                    const dataUrl = URL.createObjectURL(response.data);
                    return dataUrl;
                } catch (error) {
                    console.error("Error fetching image:", error);
                }
           
        },

                  /**
     * Function to give back a scheduling constraint url
     * @param {string} station - The Station
     * @param {string} schedulingUnit - the Unit we are looking at
     * @param {string} onDate - 2023-02-14 formatedf formated of the date
     * @returns string  with the url
     */
         getConstraintImgOnDate : async function(onDate) {
                    if (onDate===undefined) return;
                    let imageurl =   "/api/plots/daily_schedule_plot/" + onDate
                    try {
                        
                        const response = await axios.get( imageurl, {  responseType: "blob"  }  );
                        if (URL.createObjectURL) {
                            const dataUrl = URL.createObjectURL(response.data);
                            return dataUrl;
                        }
                        else
                        {
                            return response.data
                        }
                        
                    } catch (error) {
                        if (error?.response?.status==404) {
                            return;
                        }
                        else
                        {
                            console.error("getConstraintImgOnDate Error fetching image:", imageurl, error.message);
                        }
                    }
               
            },
    getExpandedSchedulingUnit: async function (type, id, expand, fields) {
        let schedulingUnit = null;
        try {
            let api = `/api/scheduling_unit_${type}/${id}/?`;
            api += (expand === '') ? '' : 'expand=' + expand + '&';
            api += (!fields || fields === '') ? '' : 'fields=' + fields + '&';
            const response = await axios.get(api);
            schedulingUnit = response.data;
        } catch (error) {
               console.error('[schedule.services.getSchedulingUnitsExpandWithFilter]', error);
        }
        return schedulingUnit


    },
    getSchedulingUnitExtended: async function (type, id, ignoreRef) {
        if (type === "constraints") {
            type = "blueprint";
        }
        let schedulingUnit = null;
        try {
            const response = await axios.get(`/api/scheduling_unit_${type}_extended/${id}/`);
            schedulingUnit = response.data;
            if (schedulingUnit && !ignoreRef) {
                if (type === "blueprint") {
                    const schedulingUnitDraft = (await axios.get(`/api/scheduling_unit_draft/${schedulingUnit.draft_id}/`)).data;
                    schedulingUnit.draft_object = schedulingUnitDraft;
                    schedulingUnit.scheduling_set_id = schedulingUnitDraft.scheduling_set_id;
                } else {
                    // Fetch all blueprints data associated with draft to display the name
                    schedulingUnit.blueprintList = (await this.getBlueprintsByschedulingUnitId(schedulingUnit.id)).data.results;
                }
                const schedulingSet = (await axios.get(`/api/scheduling_set/${schedulingUnit.scheduling_set_id}/?expand=project`)).data;
                schedulingUnit.scheduling_set_object = schedulingSet;
                schedulingUnit.scheduling_set = schedulingSet.url;

            }
        } catch (error) {
            console.error('[schedule.services.getSchedulingUnitExtended]', error);
        }
        return schedulingUnit;
    },
    /**
     * This function fetches all required details in the time range to be shown in the list with reference fields populated in the server side.
     * @param {string} startTime - start time of the range in 'YYYY-MM-DD HH:mm:ss' format
     * @param {string} endTime - end time of the range in 'YYYY-MM-DD HH:mm:ss' format
     * @returns Array - List of SUB with expanded and limited fields fetched
     */
    getExpandedSUList: async (startTime, endTime) => {
        let blueprints = [];
        try {
            let url = `/api/scheduling_unit_blueprint/?ordering=name&expand=${SU_EXPAND_FIELDS.join()}&fields=${SU_FETCH_FIELDS.join()}`;
            url = `${url}&scheduled_start_time_before=${endTime || ''}&scheduled_stop_time_after=${startTime || ''}`;
            let initialResponse = await axios.get(url);
            const totalCount = initialResponse.data.count;
            const initialCount = initialResponse.data.results.length
            blueprints = blueprints.concat(initialResponse.data.results);
            if (totalCount > initialCount) {
                url = `${url}&limit=${totalCount - initialCount}&offset=${initialCount}`;
                let secondResponse = await axios.get(url);
                blueprints = blueprints.concat(secondResponse.data.results);
            }
            _.map(blueprints, su => {
                su.start_time = su.process_start_time;
                su.stop_time = su.process_stop_time;
                return su;
            });
        } catch (error) {
            console.error('[schedule.services.getExpandedSUList]', error);
        }
        return blueprints;
    },
    getTimelineSlimBlueprints: async (startTime, endTime, id) => {
        let blueprints = [];
        try {
            let url = `/api/scheduling_unit_blueprint_slim_with_tasks/`;
            url = `${url}?process_start_time_before=${endTime || ''}&process_stop_time_after=${startTime || ''}&id=${id || ''}`;
            let initialResponse = await axios.get(url);
            const totalCount = initialResponse.data.count;
            const initialCount = initialResponse.data.results.length
            blueprints = blueprints.concat(initialResponse.data.results);
            if (totalCount > initialCount) {
                url = `${url}&limit=${totalCount - initialCount}&offset=${initialCount}`;
                let secondResponse = await axios.get(url);
                blueprints = blueprints.concat(secondResponse.data.results);
            }
            blueprints = blueprints.filter(su => su.status.toLowerCase() !== "obsolete")
        } catch (error) {
            console.error('[schedule.services.getExpandedSUList]', error);
        }
        return blueprints;
    },
    /**
     * This function fetches all required details to be shown in the list with
     * reference fields populated in the server side for one SUB
     * @returns Object - SUB with expanded and limited fields fetched
     */
    getExpandedSUB: async (id) => {
        try {
            let response = await axios.get(`/api/scheduling_unit_blueprint/${id}/?expand=${SU_EXPAND_FIELDS.join()}&fields=${SU_FETCH_FIELDS.join()}`);
            return response.data;
        } catch (error) {
            return null
        }
    },
    //>>>>>> TODO: Remove this method by using/modifying other functions with additional parameters
    getTaskBPWithSubtaskTemplate: async function (id) {
        let result;
        try {
            result = await axios.get('/api/task_blueprint/' + id);
            if (result.data) {
                result.data.template = await TaskService.getTaskTemplate(result.data.specifications_template_id);
            }
            if (result.data) {
                let subTasks = [];
                let subTasktemplate = {}
                for (const subtaskId of result.data.subtasks_ids) {
                    const subTask = await TaskService.getSubtaskDetails(subtaskId);
                    //To avoid repeated api call for template if it has already loaded
                    if (subTasktemplate[subTask.specifications_template_id]) {
                        subTask.subTaskTemplate = subTasktemplate[subTask.specifications_template_id];
                    } else {
                        const subTaskTemplate = await TaskService.getSubtaskTemplate(subTask.specifications_template_id);
                        subTask.subTaskTemplate = subTaskTemplate;
                        subTasktemplate[subTask.specifications_template_id] = subTaskTemplate;
                    }
                    subTasks.push((subTask));
                }
                result.data.subTasks = subTasks;
            }
        } catch (error) {
            console.error('[schedule.services.getTaskBPWithSubtaskTemplate]', error);
        }
        return result;
    },
    getSchedulingUnitBlueprintById: async function (id) {
        try {
            const response = await axios.get('/api/scheduling_unit_blueprint/' + id);
            let schedulingUnit = response.data;
            if (schedulingUnit) {
                const schedulingUnitDraft = await this.getSchedulingUnitDraftById(schedulingUnit.draft_id);
                schedulingUnit.scheduling_set_id = schedulingUnitDraft.scheduling_set_id;
                schedulingUnit.scheduling_set = schedulingUnitDraft.scheduling_set;
                schedulingUnit.scheduling_set_object = schedulingUnitDraft.scheduling_set_object;
                schedulingUnit.scheduling_constraints_doc = schedulingUnitDraft.scheduling_constraints_doc;
            }
            return schedulingUnit;
        } catch (error) {
            console.error(error);
            return null;
        }
    },
    getSchedulingUnitDraftById: async function (id) {
        try {
            const schedulingUnit = (await axios.get('/api/scheduling_unit_draft/' + id)).data;
            schedulingUnit.scheduling_set_object = (await axios.get(`/api/scheduling_set/${schedulingUnit.scheduling_set_id}`)).data;
            return schedulingUnit;
        } catch (error) {
            console.error('[schedule.services.getSchedulingUnitDraftById]', error);
            return null;
        }
    },
    getSubtaskOutputDataproduct: async function (id) {
        try {
            const url = `/api/subtask/${id}/output_dataproducts/`;
            const response = await axios.get(url);
            return response.data;
        } catch (error) {
            console.error('[data.product.getSubtaskOutputDataproduct]', error);
        }
    },
//>>>>>> TODO: Remove this method by using/modifying other functions with additional parameters
    getTaskBPWithSubtaskTemplateOfSU: async function (scheduleunit) {
        // there no single api to fetch associated task_blueprint, so iterate the task_blueprint id to fetch associated task_blueprint
        let taskblueprintsList = [];
        if (scheduleunit.task_blueprints_ids) {
            for (const id of scheduleunit.task_blueprints_ids) {
                await this.getTaskBPWithSubtaskTemplate(id).then(response => {
                    let taskblueprint = response.data;
                    taskblueprint['tasktype'] = 'Blueprint';
                    taskblueprint['actionpath'] = '/task/view/blueprint/' + taskblueprint['id'];
                    taskblueprint['blueprint_draft'] = taskblueprint['draft'];
                    taskblueprint['relative_start_time'] = 0;
                    taskblueprint['relative_stop_time'] = 0;
                    taskblueprint.duration = moment.utc((taskblueprint.duration || 0) * 1000).format('HH:mm:ss');
                    taskblueprintsList.push(taskblueprint);
                })
            }
        }
        return taskblueprintsList;
    },
    getTasksBySchedulingUnit: async function (id, loadTemplate, loadSubtasks, loadSubtaskTemplate) {
        let scheduletasklist = [];
        // let taskblueprints = [];
        // Common keys for Task and Blueprint
        let commonkeys = ['id', 'created_at', 'description', 'short_description', 'name', 'tags', 'updated_at', 'url', 'do_cancel', 'relative_start_time', 'relative_stop_time', 'start_time', 'stop_time', 'duration', 'status'];
        // await this.getBlueprintRelation().then( blueprints =>{
        //     taskblueprints = blueprints.data.results;'
        // });
        await this.getTasksDraftBySchedulingUnitId(id)
            .then(async (response) => {
                for (const task of response.data.results) {
                    let scheduletask = [];
                    scheduletask['tasktype'] = 'Draft';
                    scheduletask['actionpath'] = '/task/view/draft/' + task['id'];
                    scheduletask['blueprint_draft'] = task['task_blueprints'];
                    scheduletask['status'] = task['status'];

                    //fetch task draft details
                    for (const key of commonkeys) {
                        scheduletask[key] = task[key];
                    }
                    scheduletask['created_at'] = moment(task['created_at'], moment.ISO_8601).format("YYYY-MM-DD HH:mm:ss");
                    scheduletask['updated_at'] = moment(task['updated_at'], moment.ISO_8601).format("YYYY-MM-DD HH:mm:ss");
                    scheduletask['specifications_doc'] = task['specifications_doc'];
                    scheduletask.duration = moment.utc((scheduletask.duration || 0) * 1000).format('HH:mm:ss');
                    scheduletask.produced_by = task.produced_by;
                    scheduletask.produced_by_ids = task.produced_by_ids;
                    scheduletask.relative_start_time = moment.utc(scheduletask.relative_start_time * 1000).format('HH:mm:ss');
                    scheduletask.relative_stop_time = moment.utc(scheduletask.relative_stop_time * 1000).format('HH:mm:ss');
                    if (loadTemplate) {
                        scheduletask.template = await TaskService.getTaskTemplate(task.specifications_template_id);
                        scheduletask.type_value = scheduletask.template.type_value;
                    }
                    //Fetch blueprint details for Task Draft
                    const draftBlueprints = await TaskService.getDraftsTaskBlueprints(task.id);
                   

                    let subTasktemplate = {}
                    for (const blueprint of draftBlueprints) {
                        let taskblueprint = [];
                        taskblueprint['tasktype'] = 'Blueprint';
                        taskblueprint['actionpath'] = '/task/view/blueprint/' + blueprint['id'];
                        taskblueprint['blueprint_draft'] = blueprint['draft'];
                        taskblueprint['status'] = blueprint['status'];

                        for (const key of commonkeys) {
                            taskblueprint[key] = blueprint[key];
                        }
                        taskblueprint['created_at'] = moment(blueprint['created_at'], moment.ISO_8601).format("YYYY-MM-DD HH:mm:ss");
                        taskblueprint['updated_at'] = moment(blueprint['updated_at'], moment.ISO_8601).format("YYYY-MM-DD HH:mm:ss");
                        taskblueprint.duration = moment.utc((taskblueprint.duration || 0) * 1000).format('HH:mm:ss');
                        taskblueprint.relative_start_time = moment.utc(taskblueprint.relative_start_time * 1000).format('HH:mm:ss');
                        taskblueprint.relative_stop_time = moment.utc(taskblueprint.relative_stop_time * 1000).format('HH:mm:ss');
                        if (loadTemplate) {
                            taskblueprint.template = scheduletask.template;
                        }
                        if (loadSubtasks) {
                            let subTasks = [];
                            for (const subtaskId of blueprint.subtasks_ids) {
                                const subTask = await TaskService.getSubtaskDetails(subtaskId);
                                if (loadSubtaskTemplate) {
                                    //To avoid repeated api call for template if it has already loaded
                                    if (subTasktemplate[subTask.specifications_template_id]) {
                                        subTask.subTaskTemplate = subTasktemplate[subTask.specifications_template_id];
                                    } else {
                                        const subTaskTemplate = await TaskService.getSubtaskTemplate(subTask.specifications_template_id);
                                        subTask.subTaskTemplate = subTaskTemplate;
                                        subTasktemplate[subTask.specifications_template_id] = subTaskTemplate;
                                    }
                                }
                                subTasks.push((subTask));
                            }
                            taskblueprint.subTasks = subTasks;
                        }
                        //Add Blue print details to array
                        scheduletasklist.push(taskblueprint);
                    }
                    //Add Task Draft details to array
                    scheduletasklist.push(scheduletask);
                }
                //Ingest Task Relation
                if (loadTemplate) {
                    const ingest = scheduletasklist.find(task => task.template.type_value === 'ingest' && task.tasktype.toLowerCase() === 'draft');
                    const promises = [];
                    ingest.produced_by_ids.forEach(id => promises.push(this.getTaskRelation(id)));
                    const response = await Promise.all(promises);
                    response.forEach(producer => {
                        const tasks = scheduletasklist.filter(task => producer.producer_id === task.id);
                        tasks.forEach(task => {
                            task.canIngest = true;
                        });
                    });
                }
            }).catch(function (error) {
                console.error('[schedule.services.getScheduleTasksBySchedulingUnitId]', error);
            });
        return scheduletasklist;
    },
    getTaskRelation: async function (id) {
        let res;
        await axios.get(`/api/task_relation_draft/${id}`)
            .then(response => {
                res = response;
            }).catch(function (error) {
                console.error('[schedule.services.getBlueprintRelation]', error);
            });
        return res.data;
    },
    getBlueprintRelation: async function (id) {
        let res;
        await axios.get(`/api/task_relation_blueprint/${id}`)
            .then(response => {
                res = response;
            }).catch(function (error) {
                console.error('[schedule.services.getBlueprintRelation]', error);
            });
        return res.data;
    },

    addTaskRelation: async function (params, name, type) {
        try {
            let url = ''
            if (type === 'draft') {
                url = '/api/task_relation_draft/';
            } else if (type === 'blueprint') {
                url = '/api/task_relation_blueprint/';
            }
            const response = await axios.post((url), params);
            return {
                status: true,
                action: 'Added',
                'name': name,
                'data_format': params.dataformat_value,
                'msg': response.data
            };
        } catch (error) {
            return {
                status: false,
                action: 'Unable to Add',
                'name': name,
                'msg': error.response.data
            };
        }
    },
    removeTaskRelation: async function (id, name, type) {
        try {
            let url = ''
            if (type === 'draft') {
                url = `/api/task_relation_draft/${id}/`;
            } else if (type === 'blueprint') {
                url = `/api/task_relation_blueprint/${id}/`;
            }
            await axios.delete(url);
            return {
                'name': name,
                action: 'Removed',
                'status': true,
                msg: id
            };
        } catch (error) {
            return {
                'name': name,
                action: 'Unable to Remove',
                'status': false,
                msg: error.response.data
            };
        }
    },
    getTasksDraftBySchedulingUnitId: async function (id, loadTemplate) {
        let res = [];
        await axios.get('/api/scheduling_unit_draft/' + id + '/task_draft/?ordering=id')
            .then(async response => {
                res = response;
                if (response?.data?.results && loadTemplate) {
                    for (const task of response.data.results) {
                        task.template = await TaskService.getTaskTemplate(task.specifications_template_id);
                    }
                }
            }).catch(function (error) {
                console.error('[schedule.services.getTasksDraftBySchedulingUnitId]', error);
            });
        return res;
    },
    getBlueprintByschedulingUnitId: async function (id, expandFields) {
        let res = [];
        await axios.get('/api/scheduling_unit_blueprint/' + id + '?expand=' + expandFields)
            .then(response => {
                res = response.data
            }).catch(function (error) {
                console.error('[schedule.services.getBlueprintWithDraftAndTasksByschedulingUnitId]', error);
            });
        return res;
    },
    getSchedulingSets: async function (filter) {
        try {
            let schedulingSet = [];
            const fields = 'id,name,description,url,project,project_id,url';
            let response = await axios.get(`/api/scheduling_set/?${filter ? filter : ''}&fields=${fields}&ordering=name`);
            schedulingSet = schedulingSet.concat(response.data.results);
            const totalCount = response.data.count;
            const initialCount = response.data.results.length;
            if (totalCount > initialCount) {
                response = await axios.get(`/api/scheduling_set/?${filter ? filter : ''}&fields=${fields}&ordering=name&limit=${totalCount - initialCount}&offset=${initialCount}`);
                schedulingSet = schedulingSet.concat(response.data.results);
            }
            return schedulingSet;
        } catch (error) {
            console.error(error);
            return [];
        }
    },
    getSchedulingSet: async function (id, fields) {
        let res = [];
        await axios.get('/api/scheduling_set/' + id + '/' + (fields ? '?fields=' + fields : ''))
            .then(response => {
                res = response.data
            }).catch(function (error) {
                console.error('[schedule.services.getBlueprintByschedulingUnitId]', error);
            });
        return res;
    },
    getObservationStrategies: async function (filter) {
        try {
            let strategies = [];
            let response = await axios.get(`/api/scheduling_unit_observing_strategy_template/?${filter ? filter : ''}ordering=name,version`);
            strategies = strategies.concat(response.data.results);
            const totalCount = response.data.count;
            const initialCount = response.data.results.length;
            if (totalCount > initialCount) {
                response = await axios.get(`/api/scheduling_unit_observing_strategy_template/?${filter ? filter : ''}ordering=name,version&limit=${totalCount - initialCount}&offset=${initialCount}`);
                strategies = strategies.concat(response.data.results);
            }
            _.map(strategies, (strategy) => {
                strategy.uniqueId = `${strategy.name} (${strategy.purpose_value}, ${strategy.state_value}, v${strategy.version})`;
                return strategy;
            });
            return strategies;
        } catch (error) {
            console.error(error);
            return [];
        }
    },

    getSchedulingConstraintTemplates: async function () {
        let templates = [];
        try {

            let url = '/api/scheduling_constraints_template/'
            let response = await axios.get(url);
            const totalCount = response.data.count;
            const initialCount = response.data.results.length;
            templates = response.data.results;
            if (totalCount > initialCount) {
                url = `${url}?limit=${totalCount - initialCount}&offset=${initialCount}`;
                response = await axios.get(url);
                templates = templates.concat(response.data.results);
            }
        } catch (error) {
            console.error(error);
        }

        return templates
    },

    getSchedulingConstraintTemplateById: async function (id) {
        let res = [];
        await axios.get("/api/scheduling_constraints_template/" + id)
            .then(response => {
                res = response.data
            }).catch(function (error) {
                console.error('[schedule.services.getSchedulingConstraintTemplateById]', error);
            });
        return res;
    },

    saveSUDraftFromObservStrategy: async function (observStrategy, schedulingUnit, constraint) {
        try {
            // Create the scheduling unit draft with observation strategy and scheduling set
            const url = `/api/scheduling_unit_observing_strategy_template/${observStrategy.id}/create_scheduling_unit/?scheduling_set_id=${schedulingUnit.scheduling_set_id}&name=${encodeURIComponent(schedulingUnit.name)}&description=${encodeURIComponent(schedulingUnit.description)}`
            const suObsResponse = await axios.post(url);
            if (schedulingUnit.rank !== '') {
                suObsResponse.data.rank = schedulingUnit.rank;
            }
            if (schedulingUnit.priority_queue !== '') {
                suObsResponse.data.priority_queue = schedulingUnit.priority_queue;
            }
            suObsResponse.data['output_pinned'] = schedulingUnit.output_pinned;
            schedulingUnit = suObsResponse.data;
            if (schedulingUnit && schedulingUnit.id) {
                // Update the newly created SU draft constraints and task drafts with captured parameter values
                schedulingUnit.scheduling_constraints_doc = constraint.scheduling_constraints_doc;
                schedulingUnit.scheduling_constraints_template_id = constraint.id;
                schedulingUnit.scheduling_constraints_template = constraint.constraint.url;
                let suTaskDrafts = (await axios.get(`/api/scheduling_unit_draft/${schedulingUnit.id}/?expand=task_drafts&fields=task_drafts`)).data.task_drafts;
                // Update only the tasks for which parameters are captured
                let tasksToUpdate = {};
                for (const param of observStrategy.template.parameters) {
                    const refs = param.refs[0].split('/');
                    if (refs[1] === 'tasks') {
                        tasksToUpdate[refs[2]] = refs[2];
                    }
                }
                await this.updateSUDraftFromObservStrategy(observStrategy, schedulingUnit, suTaskDrafts, tasksToUpdate);
                if (schedulingUnit.task_drafts.length > 0) {
                    schedulingUnit['isSUUpdated'] = true;
                    schedulingUnit['taskName'] = '(Tasks)';
                    return schedulingUnit;
                }
            } else {
                return {
                    error: true,
                    messsage: 'Unable to create Scheduling Unit'
                };
            }
        } catch (error) {
            console.error(error);
            appGrowl.show({
                severity: 'error',
                summary: 'Error',
                sticky: 'true',
                detail: <ShowErrorDetails response={error}/>
            });
            return {
                error: true,
                message: 'Unable to create Scheduling Unit'
            };
        }

    },

    updateSUDraftFromObservStrategy: async function (observStrategy, schedulingUnit, tasks, tasksToUpdate) {
        let taskName = '';
        const outputPinned = schedulingUnit.output_pinned;
        try {
            delete schedulingUnit['duration'];
            schedulingUnit['isSUUpdated'] = false;
            schedulingUnit = await this.updateSchedulingUnitDraft(schedulingUnit);
            if (!schedulingUnit.error) {
                schedulingUnit['isSUUpdated'] = true;
                if (outputPinned !== null) {
                    for (let task of tasks) {
                        if (!_.includes(tasksToUpdate, task.name)) {
                            const id = task.id;
                            task['output_pinned'] = outputPinned;
                            task = await TaskService.updateTask('draft', task);
                            if (task === null) {
                                console.error(`Unable to update the task details "Output_Pinned" for Task Id [${id}]`);
                            }
                        }
                    }
                }

                for (const taskToUpdate in tasksToUpdate) {
                    let task = tasks.find(task => {
                        return task.name === taskToUpdate
                    });
                    taskName = taskToUpdate;
                    if (task) {
                        const taskUpdateParam = observStrategy.template.tasks[taskToUpdate];
                        const taskKeys = Object.keys(taskUpdateParam)
                        for (const taskKey of taskKeys) {
                            if (taskKey !== 'specifications_template') { // to retain the 'specifications_template' value
                                task[taskKey] = taskUpdateParam[taskKey];
                            }
                        }
                        if (task.specifications_doc.station_configuration?.station_groups) {
                            task.specifications_doc.station_configuration.station_groups = taskUpdateParam.specifications_doc.station_configuration.station_groups
                        }
                        delete task['duration'];
                        delete task['relative_start_time'];
                        delete task['relative_stop_time'];
                        if (outputPinned !== null) {
                            task['output_pinned'] = outputPinned;
                        }
                        task = await TaskService.updateTask('draft', task);
                        if (task.error) {
                            schedulingUnit = task;
                        }
                    } else {
                        return {
                            taskName: taskName,
                            error: true,
                            message: 'Unable to Update Task Drafts'
                        }
                    }
                }
            } else {
                schedulingUnit['isSUUpdated'] = false;
            }
            schedulingUnit['taskName'] = taskName;
            return schedulingUnit;
        } catch (error) {
            console.error(error);
            schedulingUnit['isSUUpdated'] = false;
            return {
                taskName: taskName,
                error: true,
                message: 'Unable to Update Task Drafts'
            }
        }
    },
    updateSchedulingUnitDraft: async function (schedulingUnit) {
        let suUpdateResponse = {};
        try {
            schedulingUnit.scheduling_constraints_doc = (schedulingUnit.scheduling_constraints_doc == null) ? "" : schedulingUnit.scheduling_constraints_doc;
            suUpdateResponse = await axios.put(`/api/scheduling_unit_draft/${schedulingUnit.id}/`, schedulingUnit);
            suUpdateResponse['message'] = 'Success';
            suUpdateResponse['isSUUpdated'] = true;
            return suUpdateResponse;
        } catch (error) {
            suUpdateResponse['message'] = 'Unable to create Scheduling Unit';
            suUpdateResponse['isSUUpdated'] = false;
            console.error("[schedule.service.updateSchedulingUnitDraft]", error);
            return suUpdateResponse
        }
    },

    getSchedulingListByStatus: async function (status) {
        try {
            let url = `/api/scheduling_unit_blueprint/?ordering=name&expand=${SU_EXPAND_FIELDS.join()}&fields=${SU_FETCH_FIELDS.join()}`
            url = url + `&status=${status}`
            const response = await axios.get(url);
            return response.data.results;
        } catch (error) {
            console.error(error);
            return [];
        }
    },
    /**
     * To fetch the scheduling unit drafts belonging to the Scheduling Set.
     * @param {Number} id - Scheduling set id
     * @param {String} fields - String of properties to be fetched separated by comma (eg: id,name,...). If no value passed, all fields will be fetched.
     * @param expand
     * @returns Array of scheduling unit drafts
     */
    getSchedulingBySet: async function (id, fields, expand) {
        let suDrafts = [];
        try {
            const initialResponse = await axios.get(`/api/scheduling_set/${id}/scheduling_unit_draft/?ordering=id${expand ? ('&expand=' + expand) : ''}${fields ? ('&fields=' + fields) : ''}`);
            const totalCount = initialResponse.data.count;
            const initialCount = initialResponse.data.results.length
            suDrafts = suDrafts.concat(initialResponse.data.results);
            if (totalCount > initialCount) {
                let secondResponse = await axios.get(`/api/scheduling_set/${id}/scheduling_unit_draft/?ordering=id&limit=${totalCount - initialCount}&offset=${initialCount}${expand ? ('&expand=' + expand) : ''}${fields ? ('&fields=' + fields) : ''}`);
                suDrafts = suDrafts.concat(secondResponse.data.results);
            }
            return suDrafts;
        } catch (error) {
            console.error('[project.services.getSchedulingUnitBySet]', error);
        }
    },
    createSchedulingUnitBlueprintTree: async function (id) {
        try {
            const response = await axios.post(`/api/scheduling_unit_draft/${id}/create_scheduling_unit_blueprint_and_tasks_and_subtasks_tree`);
            return response.data;
        } catch (error) {
            console.error(error);
            return null;
        }
    },
    copySchedulingUnitDraft: async function (id, param) {
        try {
            return await axios.post(`/api/scheduling_unit_draft/${id}/copy`, param);
        } catch (error) {
            return error.response
        }
    },
    copySchedulingUnitBluePrint: async function (id, param) {
        try {
            return await axios.post(`/api/scheduling_unit_blueprint/${id}/copy_to_draft`, param);
        } catch (error) {
            console.error(error);
            return error.response
        }
    },
    getStationGroup: async function () {
        try {
            const commonSchemaTemplate = await UtilService.getCommonSchemaTemplate();
            let stationGroupName = [];
            const stationGroups = commonSchemaTemplate.ref_resolved_schema.definitions.station_group.anyOf;
            for (const obj of stationGroups) {
                if (obj.title !== 'Custom') {
                    stationGroupName.push({value: obj.title});
                }
            }
            return stationGroupName;
        } catch (error) {
            console.error(error);
            return [];
        }
    },
    getStations: async function (group) {
        let stationGroup = {};
        try {
            const commonSchemaTemplate = await UtilService.getCommonSchemaTemplate();
            const stationGroups = commonSchemaTemplate.ref_resolved_schema?.definitions?.station_group.anyOf;
            const stationGroupObj = _.find(stationGroups, {title: group});
            if (stationGroupObj) {
                stationGroup = stationGroupObj?.default;
            }
        } catch (error) {
            console.error(error);
            return [];
        }
        stationGroup['group'] = group;
        return stationGroup;
    },
    // To get stations of main groups
    getMainGroupStations: async function () {
        let stationGroups = {};
        try {
            await Promise.all(
                [this.getStations('Core'), this.getStations('Remote'), this.getStations('International')]
            ).then(async (results) => {
                for (const result of results) {
                    stationGroups[result.group] = result.stations;
                }
            });
        } catch (error) {
            console.error(error);
        }
        return stationGroups;
    },
    getProjectList: async function () {
        try {
            const response = await axios.get('/api/project/?limit=150');
            return response.data.results;
        } catch (error) {
            console.error('[project.services.getProjectList]', error);
        }
    },
    saveSchedulingSet: async function (suSet) {
        try {
            const response = await axios.post(('/api/scheduling_set/'), suSet);
            return response.data;
        } catch (error) {
            console.log(error.response.data);
            return error.response.data;
        }
    },
    /**
     * Delete Scheduling Unit based on type
     * @param {*} type
     * @param {*} id
     */
    deleteSchedulingUnit: async function (type, id) {
        try {
            const url = type.toLowerCase() === 'blueprint' ? `/api/scheduling_unit_blueprint/${id}` : `/api/scheduling_unit_draft/${id}`;
            await axios.delete(url);
            return true;
        } catch (error) {
            console.error(error);
            return false;
        }
    },
    
       getSchedulingUnitFilterDefinition: async function (type) {
        
        if (type === "constraints") {
            type = "blueprint";
        }

        if (type.startsWith("draft"))  { type = "draft";  }

        let res = [];
        try {
            res = await axios.options(`/api/scheduling_unit_${type}_extended/`);
        } catch (error) {
            console.error('[schedule.services.getSchedulingUnitFilterDefinition]', error);
        }
        
        return res;
    },
    /* get All Task Relation */
    getAllTaskRelation: async function (type, filter) {
        try {
            let url = ''
            if (type === 'draft') {
                url = `/api/task_relation_draft/?${filter}`;
            } else if (type === 'blueprint') {
                url = `/api/task_relation_blueprint/?${filter}`;
            }
            const response = await axios.get(url);
            return response.data.results;
        } catch (error) { /* empty */ }
    },
    /* Create Task Relation based on consumer(Ingest) and producer */
    createTaskRelation: async function (taskRelDraftObj, obj, type) {
        let taskRelDraftPromises = [];
        try {
            if (taskRelDraftObj) {
                taskRelDraftObj.forEach((tObj) => {
                    taskRelDraftPromises.push(this.addTaskRelation(tObj, tObj.name, type));
                });
                return await Promise.all(taskRelDraftPromises);
            }
        } catch (error) {
            console.log(error.response.data);
            return error.response.data;
        }
    },
    /* Delete Task Relation based on consumer(Ingest) and producer */
    deleteTaskRelation: async function (taskRelDelDraft, obj, type) {
        let taskRelDelDraftPromises = [];
        try {
            if (taskRelDelDraft) {
                taskRelDelDraft.forEach((tObj) => {
                    taskRelDelDraftPromises.push(this.removeTaskRelation(tObj.id, tObj.producer.name, type));
                });
                return await Promise.all(taskRelDelDraftPromises);
            }
        } catch (error) {
            console.log(error.response.data);
            return error.response.data;
        }
    },
    getTaskRelationDraftAPIReqObj: async function (taskRelationTemplateName) {
        let taskRelationDraftObj = {
            "consumer": "/api/task_draft",
            "dataformat": "/api/dataformat/MeasurementSet",
            "input_role": "/api/task_connector_type",
            "output_role": "/api/task_connector_type",
            "producer": "/api/task_draft",
            "related_task_relation_blueprint": [],
            "selection_doc": {},
            // "selection_template": "/api/task_relation_selection_template/1",
            "tags": []
        }
        try {
            const response = await axios.get(`/api/task_relation_selection_template/?ordering=-id&name=${taskRelationTemplateName}`);
            const taskRelTemplates = response.data.results;
            let taskRelTemplate = _.find(taskRelTemplates, {"state_value": "active"});
            taskRelTemplate = taskRelTemplate || (taskRelTemplates.length > 0 ? taskRelTemplates[0] : {});
            taskRelationDraftObj.selection_template = taskRelTemplate.url;
        } catch (error) {
            console.error("schdeule.services.taskRelationDraftAPIReqObj", error);
        }
        return taskRelationDraftObj;
    },
    cancelSchedulingUnit: async (id) => {
        let cancelledSU = null;
        try {
            const url = `/api/scheduling_unit_blueprint_extended/${id}/cancel/`;
            cancelledSU = (await axios.post(url, {})).data;
        } catch (error) {
            console.error(error);
        }
        return cancelledSU;
    },
    getSchedulingUnitsExpandWithFilter: async function (type, expand, filters, orderBy, limit, offset, fields) {
        if (type === "constraints") {
            type = "blueprint";
        }
        if (type.startsWith("draft")) type="draft";

        let response = null;
        let api = `/api/scheduling_unit_${type}${fields ? '' : '_extended'}/?`;
        try {
            api += (expand === '') ? '' : 'expand=' + expand + '&';
            api += (!fields || fields === '') ? '' : 'fields=' + fields + '&';
            api += (filters === '') ? '' : filters + '&';
            api += (orderBy === '') ? '' : orderBy + '&';
            api += 'limit=' + limit + '&offset='
            api += (offset === '') ? 0 : offset;
            response = await axios.get(api);

        } catch (error) {
            console.info('[schedule.services.getSchedulingUnitsExpandWithFilter]', api,error?.message);
        }
        return response;
    },
    createSUCleanupTask: async (schedulingUnit) => {
        let response = {};
        try {
            const url = `/api/scheduling_unit_blueprint/${schedulingUnit.id}/create_cleanuptask_for_scheduling_unit_blueprint/`;
            response = (await axios.post(url, {}));
            response['message'] = 'Success';
        } catch (error) {
            const errorData = error.response.data; // To be utilized to create custom error message.
            response['message'] = `Failed due to error - ${errorData.substring(0, errorData.indexOf('Request Method:'))}`;
            console.error(error);
        }
        return response;
    },
    setConstraintsEditorOutputService: async (req) => {
        let response = {};
        try {
            const url = `/api/scheduling_unit_blueprint/${req.id}/`;
            response = (await axios.put(url, req));
            response['message'] = 'Success';
        } catch (error) {
            const errorData = error.response.data; // To be utilized to create custom error message.           
            response['message'] = `Failed due to error - ${errorData.substring(0, errorData.indexOf('Request Method:'))}`;
        }
        return response;
    },
    getConstraintsEditorOutputService: async (req) => {
        let response = {};
        try {
            const url = `/api/scheduling_unit_blueprint/${req}/`;
            response = (await axios.get(url));
        } catch (error) {
            const errorData = error.response.data; // To be utilized to create custom error message.
            response['message'] = `Failed due to error - ${errorData.substring(0, errorData.indexOf('Request Method:'))}`;
            console.error(error);
        }
        return response.data;
    },
    updateSchedulingUnit: async function (type, schedulingUnit) {
        try {
            let url = `/api/scheduling_unit_${type}/${schedulingUnit.id}/`;
            const response = await axios.patch(url, schedulingUnit);
            return response.data;
        } catch (error) {
            console.error('[schedule.services.updateSchedulingUnit]', "Error: ", error);
            return null
        }
    },
    getSUDataproducts: async function (id) {
        let response = {};
        try {
            let url = `/api/scheduling_unit_blueprint/${id}/?expand=task_blueprints,task_blueprints.subtasks,task_blueprints.subtasks.output_dataproducts` +
                `&fields=task_blueprints.id,task_blueprints.subtasks.id,task_blueprints.subtasks.output_dataproducts.feedback_doc`;
            response = await axios.get(url);
            return response.data;

        } catch (error) {
            console.error('[schedule.services.getSUDataproduct]', error);
            return null;
        }
    },
    copySUSpecAndFailedTasks: async (schedulingUnit) => {
        let response = {};
        try {
            const url = `/api/scheduling_unit_blueprint/${schedulingUnit.id}/copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft/`;
            response = (await axios.post(url, {}));
            response['message'] = 'Success';
        } catch (error) {
            const errorData = error.response.data;
            response['message'] = `Failed due to error - ${errorData.substring(0, errorData.indexOf('Request Method:'))}`;
            console.error('[schedule.services.copySUSpecAndFailedTasks]', "Error: ", error);
        }
        return response;
    },
    copySUFailedTasks: async (schedulingUnit) => {
        let response = {};
        try {
            const url = `/api/scheduling_unit_blueprint/${schedulingUnit.id}/create_copies_of_failed_tasks_via_draft/`;
            response = (await axios.post(url, {}));
            response['message'] = 'Success';
        } catch (error) {
            const errorData = error.response.data;
            response['message'] = `Failed due to error - ${errorData.substring(0, errorData.indexOf('Request Method:'))}`;
            console.error('[schedule.services.copySUFailedTasks]', "Error: ", error);
        }
        return response;
    },
    updateSUBTasks: async (schedulingUnit) => {
        let response = {};
        try {
            const url = `/api/scheduling_unit_blueprint/${schedulingUnit.id}/update_task_blueprints_and_subtasks_graph_from_draft/`;
            response = (await axios.post(url, {}));
            response['message'] = 'Success';
        } catch (error) {
            const errorData = error.response.data;
            response['message'] = `Failed due to error - ${errorData.substring(0, errorData.indexOf('Request Method:'))}`;
            console.error('[schedule.services.updateSUBTasks]', "Error: ", error);
        }
        return response;
    },
    /**
     * To get scheduling unit blueprint with limited details required to populate in workflow list
     * @param {Number} id - scheduling unit blueprint id
     * @returns  scheduling unit blueprint object
     */
    getWorkflowSU: async (id) => {
        let response = {}
        try {
            const url = `/api/scheduling_unit_blueprint/?id=${id}&expand=draft,draft.scheduling_set&fields=id,name,status,unschedulable_reason,draft.scheduling_set.project_id`
            response = await axios.get(url);
        } catch (error) {
            console.error('[schedule.services.getWorkflowSU]', error);
        }
        return response.data?.results;
    },
    getStrategyPurpose: async () => {
        let response = [];
        try {
            const url = `/api/template_purpose`
            response = (await axios.get(url)).data.results;
        } catch (error) {
            console.error('[schedule.services.template_purpose]', error);
        }
        return response;
    },
    getStrategyState: async () => {
        let response = [];
        try {
            const url = `/api/template_state`
            response = (await axios.get(url)).data.results;
        } catch (error) {
            console.error('[schedule.services.template_state]', error);
        }
        return response;
    },
    updateSchedulingUnitValue: async (type, schedulingUnit, tasks) => {
        try {
            let response;
            if (type !== 'draft') {
                let url = `/api/scheduling_unit_${type}/${schedulingUnit.id}/`;
                response = await axios.patch(url, schedulingUnit);
            } else {
                response = {data: schedulingUnit};
            }
            if (type === 'draft' && tasks) {
                for (const task of tasks) {
                    await TaskService.updateTaskValue(task);
                }
            }
            return response.data;
        } catch (error) {
            console.error('[schedule.services.updateSchedulingUnitValue]', "Error: ", error);
            return null;
        }
    },
    saveSchedulingUnit: async (su, strategyParameter) => {
        let response = {};
        try {//output_pinned: since it is not updating task directly and also not used in grid, so it is hold and based on api changes it will update 
            let url = `/api/scheduling_unit_observing_strategy_template/${su.observation_strategy_template_id}/create_scheduling_unit/?scheduling_set_id=${su.scheduling_set_id}&name=${encodeURIComponent(su.name)}&description=${encodeURIComponent(su.description)}&rank=${su.rank}`;
            if (su.priority_queue) {
                url += `&priority_queue=${su.priority_queue}`;
            }
            response = (await axios.post(url, strategyParameter));
            response['message'] = 'Success';
            response['isSUUpdated'] = true;
        } catch (error) {
            response['isSUUpdated'] = false;
            response['message'] = error.response.data;
            console.error('[schedule.services.saveSU]', "Error: ", error);
        }
        return response;
    },
    validateSpecificationsDoc: async function (template, specification_doc) {
        try {
            const url = `${template}/validate_specifications_doc`;
            const response = await axios.put(url, {'specifications_doc': specification_doc});
            if (!response.data['valid']) {
                console.error(response.data['message']);
            }
            return response.data;
        } catch (error) {
            console.error(error);
        }
    },
    ScheduleBluePrintOn: async function (id, startTime) {
        const  param = {
            start_time:startTime 
        }
        try {
            return await axios.post(`/api/scheduling_unit_blueprint/${id}/schedule`, param);
        } catch (error) {
            console.error(error);
            return error.response
        }
    },
    
}

export default ScheduleService;
