import { render, act,  fireEvent,screen  } from '@testing-library/react';
import {  MockAxiosSecurity } from './axiosmocks.';
import MockAdapter from 'axios-mock-adapter';
import axios from 'axios';


import DynamicScheduler from '../components/DynamicScheduler';
import PermissionStackUtil from '../authenticate/permission.stack.handler';
import { remove404Errors } from '../utils/test.helper';


function MakePrimaryMock(schedularvalue =true,saveStatusCode=200) {
  let mock = new MockAdapter(axios);
  mock.onGet("/api/subsystem/scheduler").reply(200,{   "name": "scheduler",   "url": "https://tmss.lofar.eu/api/subsystem/scheduler",   "created_at": "2022-10-03T08:55:11.449584",    "description": "The scheduler subsystem takes care of scheduling all scheduling_units taking into account all their constraints and station availability",    "status": "https://tmss.lofar.eu/api/subsystem_status/active",    "status_value": "active",    "tags": []} );  
  mock.onGet("/api/setting/fixed_time_scheduling_enabled").reply(200,{ value:schedularvalue});  
  mock.onGet("/api/setting/dynamic_scheduling_enabled").reply(200,{  value: schedularvalue});  
  mock.onPatch("/api/setting/dynamic_scheduling_enabled").reply(saveStatusCode,{value:!schedularvalue});
  mock.onPatch("/api/setting/fixed_time_scheduling_enabled").reply(saveStatusCode,{value:!schedularvalue});
  
  return mock;
}
describe('DynamicScheduler ', () => {
  it('Renders DynamicScheduler  without Security , With Data', async () => {
    MockAxiosSecurity(MakePrimaryMock(true));
    remove404Errors();
    let component
    await act(async () => { component = render(<DynamicScheduler />); });
    expect(component.asFragment).toMatchSnapshot("After  Creation");
  });
});

describe('DynamicScheduler With proper authorization ', () => {

  it('Renders DynamicScheduler  with Data With Security', async () => {
    MockAxiosSecurity(MakePrimaryMock(false),200,true);
    await PermissionStackUtil.getPermissions(true);
    let component
    await act(async () => { component = render(<DynamicScheduler />); });
    expect(component.asFragment()).toMatchSnapshot("After Permission Creation");
  });

  it('Renders DynamicScheduler  with Data With Security schedulars not active', async () => {
   
    await DynamicTest(200,"");
   });

   it('Renders DynamicScheduler  with Data With Security schedulars not active error saving', async () => {
       await DynamicTest(500," error saving");
   });

});


async function DynamicTest(statusresult, SnapAppender) {
  MockAxiosSecurity(MakePrimaryMock(false,statusresult),true);
  await PermissionStackUtil.getPermissions(true);
  let component;
  let callbackfn = jest.fn();
  await act(async () => { component = render(<DynamicScheduler callBack= {callbackfn} />); });
  expect(component.asFragment()).toMatchSnapshot("After Permission Creation schedular not active + SnapAppender");

  let aswitch = component.queryAllByRole('switch');
  await act(async () => { fireEvent.click(aswitch[0]); });
  const dialog = screen.getByTestId('confirm_dialog');
  expect(dialog).toBeInTheDocument();
  const submitButton = screen.getByText('Yes');
  await act(async () => { fireEvent.click(submitButton); });
  expect(component.asFragment()).toMatchSnapshot("After Permission Creation With Dialog " +SnapAppender);
 

  // then fire the next switch
  await act(async () => { fireEvent.click(aswitch[1]); });
  const submitButtontwo = screen.getByText('Yes');
  await act(async () => { fireEvent.click(submitButtontwo); });
  expect(component.asFragment()).toMatchSnapshot("After Permission Creation With Dialog Fixed Schedular" +SnapAppender) ;
  if (statusresult==200)   expect(callbackfn).toHaveBeenCalled();

  // change and then cancel
  await act(async () => { fireEvent.click(aswitch[1]); });
  let cancelButtonNo = screen.getByText('No');
  await act(async () => { fireEvent.click(cancelButtonNo); });

  expect(component.asFragment()).toMatchSnapshot("After Cancel Action" +SnapAppender);
}

