

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act ,fireEvent} from '@testing-library/react';

import { MockAxiosSecurity  } from './axiosmocks.'
import { MemoryRouter } from "react-router-dom";

import PermissionStackUtil from "../authenticate/permission.stack.handler";
import DailyConstraintView from "../routes/Constraint/DailyConstraintView";


function MakePrimaryMock() {
    let mock = new MockAdapter(axios);
    let smallimage = "data:image/gif;base64,R0lGODlhAQABAAAAACwAAAAAAQABAAA="

    mock.onGet("api/plots/daily_schedule_plot/2023-12-24").reply(200, smallimage)
    

    return mock

}

describe('Daily Constraints', () => {

    it('Daily Constraints  renders correctly without proper security', async () => {
      
        MakePrimaryMock();
        let component
        await act(async () => { component = render(<MemoryRouter><DailyConstraintView  history={{}} location="/constraint/view/" /></MemoryRouter>) });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    
    it('Daily Constraints Renders correctly with  data', async () => {
        let  component  = await RenderConstraint();
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        const previousDayButton = component.getByTitle('previous day');
         // Simulate a click on the button
        fireEvent.click(previousDayButton);
        const componentSnapshotAfterClick = component.asFragment();
        expect(componentSnapshotAfterClick).toMatchSnapshot("Previous day");
        const nextDayButton = component.getByTitle('next day');
         // Simulate a click on the button
        fireEvent.click(nextDayButton);
        const componentSnapshotAfterNextDay = component.asFragment();
        expect(componentSnapshotAfterNextDay).toMatchSnapshot("Next day");
        component.unmount();
      });
});

async function RenderConstraint() {
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component;
    const match = {
        params: {
            date :"2023-12-24"
        }
    }
    await act(async () => {
        component = render(<MemoryRouter><DailyConstraintView history={{}} location="/constraint/view/" match={match} /></MemoryRouter>);
    });
    return   component ;
}

