import { render, act,waitFor } from '@testing-library/react';
import { MemoryRouter } from "react-router-dom";
import ProtectedRoute from "../components/ProtectedRouteComponent";
import { MockAxiosSecurity } from "./axiosmocks.";
import PermissionStackUtil from "../authenticate/permission.stack.handler";


import NotFound from "../layout/components/NotFound";





describe('Protected Route', () => {

    it('Renders Protected Route Without loaded userpermission without supplied permissions', async () => {

        let component = await  RenderProtectedRoute();
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });

    it('Renders Protected Route without loaded userpermissions Requiring permissions ', async () => {
        let permissions = ['task', 'list']
        let component = await  RenderProtectedRoute(permissions);
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });


    it('Renders Protected Route With USer Permission requering Permission', async () => {
        MockAxiosSecurity();
        await PermissionStackUtil.getPermissions(true);

        let permissions = ['task', 'list']
        let component = await  RenderProtectedRoute(permissions);
        await waitFor (() =>expect(component.queryAllByText('Page Not Found - 404 error').length).toBe(1),{ timeout:1000}); 
        expect(component.asFragment()).toMatchSnapshot();
                component.unmount();
    });

    it('Renders Protected Route WithoutRequiring Permissions ', async () => {
        MockAxiosSecurity();
        let permissions = []
        let component = await  RenderProtectedRoute(permissions);
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });

});



async function RenderProtectedRoute (permissions) {
    let child = NotFound // unpurpose a simple component used here. So notfound is not notfound, but the simple component :)
    let component
    await act(async () => { component = render(<MemoryRouter><ProtectedRoute  history={{}} location="/somewhere/" permissions={permissions} component={child}></ProtectedRoute></MemoryRouter>) });
    return component
}


