

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, fireEvent, waitFor } from '@testing-library/react';
import FailureReport from '../routes/Report/failure/failure.report';
import failurereport from '../__mocks__/failure.report.json'
import FailureBusinessTemplate from "../routes/Report/failure/failure.business.template";
import FailureBusinessPdf from "../routes/Report/failure/failure.business.pdf";
import { remove404Errors } from "../utils/test.helper";



function MakePrimaryMock() {
    let mock = new MockAdapter(axios);
    jest.mock('chart.js', () => ({
        ChartJS: jest.fn(),
    }));

    mock.onGet("/api/failure_report/?start=2023-01-01T00:00:00Z&stop=2024-12-31T23:59:59Z&format=json").reply(200, failurereport);
}



describe('Failure Report Render Without Contents', () => {

    it('Failure Report  renders correctly without data', async () => {
        MakePrimaryMock();
        remove404Errors();
        let component
        await act(async () => { component = render(<FailureReport />); });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();
    });
});

describe('Failure Report Render With Contents', () => {



    it('Failure Report  renders correctly when clicking generate', async () => {

        MakePrimaryMock();
        let component
        await act(async () => { component = render(<FailureReport />); });


        const generateButton = component.getByLabelText('Generate Failure Report');



        fireEvent.click(generateButton);
        const componentSnapshotAfterClick = component.asFragment();
        expect(componentSnapshotAfterClick).toMatchSnapshot("After click Generate");

        await waitFor(() => {
            expect(component.container.querySelector('.failureProgress')).toBeNull();
        }, { timeout: 10000 });

        

        component.unmount();
    });
});












describe('Failure Report Generates exports', () => {



    it('Failure Report  renders correctly when clicking generate', async () => {
        MakePrimaryMock();
        let component
        await act(async () => { component = render(<FailureReport />); });
        const generateButton = component.getByLabelText('Generate Failure Report');
        fireEvent.click(generateButton);
        await waitFor(() => { expect(component.container.querySelector('.failureProgress')).toBeNull(); }, { timeout: 5000 });

        const generatePDfButton = component.getByLabelText('Download Pdf');
        fireEvent.click(generatePDfButton);

        // Assert that jsPDF constructor was called
        const componentSnapshotAfterDataLoading = component.asFragment();
        expect(componentSnapshotAfterDataLoading).toMatchSnapshot("After pdf generating");

        component.unmount();
    });

    it('Failure Report  renders correctly when clicking json', async () => {
        MakePrimaryMock();
        let component
        await act(async () => { component = render(<FailureReport />); });
        const generateButton = component.getByLabelText('Generate Failure Report');
        fireEvent.click(generateButton);
        await waitFor(() => { expect(component.container.querySelector('.failureProgress')).toBeNull(); }, { timeout: 5000 });

        

        

        const generateJsonButton = component.getByLabelText('Download Json');
        fireEvent.click(generateJsonButton);
        await waitFor(() => {
            const anchorElement = component.container.querySelector('a[href*="data:text/json"]');
            expect(anchorElement).toBeInTheDocument();
          });
        // Assert that json download  was called
        const componentSnapshotAfterDataLoading = component.asFragment();
        expect(componentSnapshotAfterDataLoading).toMatchSnapshot("After json generating");

        component.unmount();
    });

});


describe('Failure Templating working when empty', () => {
    it('on_sky_start_time_Template', () => {
        expect(FailureBusinessTemplate.on_sky_start_time_Template(null)).toEqual("")
    });

    it('on_sky_duration_Template', () => {
        expect(FailureBusinessTemplate.on_sky_duration_Template(null)).toEqual("")
    });

    it('issues_Template', () => {
        expect(FailureBusinessTemplate.issues_Template(null)).toEqual("")
    });

    it('issuessubtypes_Template', () => {
        expect(FailureBusinessTemplate.issuessubtypes_Template(null)).toEqual("")
    });

    it('subtasks_Template', () => {
        expect(FailureBusinessTemplate.subtasks_Template(null)).toEqual("")
    });

});


describe('Failure PDf Business  working when empty', () => {
    it('writeTable', () => {
        expect(FailureBusinessPdf.writeTable(null, null)).toBeUndefined();
    });

});
