

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, waitFor } from '@testing-library/react';
import reservationList from '../__mocks__/reservation.list.json'
import reservationListLimited from '../__mocks__/reservation.list.limit.json'

import { MockAxiosSecurity  } from './axiosmocks.'
import { MemoryRouter } from "react-router-dom";
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { ReservationList } from "../routes/Reservation/reservation.list";
import  cycles from "../__mocks__/cycle.json"


function MakePrimaryMock() {
    let mock = new MockAdapter(axios);
    mock.onOptions("/api/reservation/").reply(200, reservationList);
    mock.onGet("/api/reservation/?limit=10&offset=0").reply(200, reservationListLimited);
    mock.onGet("/api/cycle").reply(200, cycles);
    mock.onGet("/api/reservation/?ordering=name&limit=10&offset=0").reply(200, reservationListLimited);
    return mock
}

describe('Reservation List ', () => {

    it('Reservation List  renders correctly without proper security', async () => {
        MakePrimaryMock();
        let component
        await act(async () => { component = render(<MemoryRouter><ReservationList  match={{}}  history={{}} location="/reservation/list/" /></MemoryRouter>) });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('Reservation List  correctly with data', async () => {
        let {  component } = await RenderReservationist();
        const componentSnapshot = component.asFragment();
       
        await waitFor(() => {
            const anchorElement = component.container.querySelector('.fa-search');
            expect(anchorElement).toBeInTheDocument();
          });

        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });
  
});



async function RenderReservationist() {
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component;
    let ListInstance;
    await act(async () => {
        component = render(<MemoryRouter><ReservationList history={{}} location="/reservation/list/" ref={(instance) => { ListInstance = instance; } } /></MemoryRouter>);
    });
    return { ListInstance, component };
}

