

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act } from '@testing-library/react';

import systemeventoptions from '../__mocks__/system_event.options.json'
import systemeventtemplate from '../__mocks__/system_event_template.json'
import systemeventtype from '../__mocks__/system_event_type.json'
import systemeventsubtype from '../__mocks__/system_event_subtype.json'
import systemeventseverity from '../__mocks__/system_event_severity.json'
import systemeventsstatus from '../__mocks__/system_event.status.json'
import systemeventlist from '../__mocks__/system_event_list.json'

import { remove404Errors } from "../utils/test.helper";
import { MockAxiosSecurity  } from './axiosmocks.'
import { MemoryRouter } from "react-router-dom";


import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { SystemEventList } from "../routes/SystemEvent/system.event.list";
import { giveConsistentNodes } from "../utils/test.helper";
import { SystemEventListLite } from "../routes/SystemEvent/system.event.listlite";



function MakePrimaryMock() {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    let mock = new MockAdapter(axios);
    mock.onOptions("/api/system_event/").reply(200, systemeventoptions);
    mock.onGet("/api/system_event_template").reply(200, systemeventtemplate);
    mock.onGet("/api/system_event_type").reply(200, systemeventtype);
    mock.onGet("/api/system_event_subtype").reply(200, systemeventsubtype);
    mock.onGet("/api/system_event_severity").reply(200, systemeventseverity);
    mock.onGet("/api/system_event_status").reply(200, systemeventsstatus);
    mock.onGet("/api/system_event/?ordering=id&limit=10&offset=0").reply(200, systemeventlist);
    mock.onGet("/api/system_event/?affected_tasks=1&affected_tasks=2&affected_tasks=3&affected_tasks=4&undefined&limit=123&offset=0").reply(200, systemeventlist);
    mock.onGet("/api/system_event/?limit=1").reply(200, systemeventlist);
    return mock

}

describe('Systen Event List ', () => {

    it('System Event List  renders correctly without security', async () => {
        new MockAdapter(axios);
       let component
       remove404Errors();
        await act(async () => { component = render(<MemoryRouter><SystemEventList  history={{}} location="/systemevent/list/" /></MemoryRouter>) });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('System Event Renders correctly with data', async () => {
        let {  component } = await Renderist();
        expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot();
        component.unmount();

    });
   
});

describe('Systen Event List Lite ', () => {
    it('System Event Renders correctly with data', async () => {
        let {  component } = await RenderistLite();
        expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot();
        component.unmount();

    });
   
});


async function Renderist() {
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component;
    let eventListInstance;
    await act(async () => {
        component = render(<MemoryRouter><SystemEventList history={{}} location="/systemevent/list/" ref={(instance) => { eventListInstance = instance; } } /></MemoryRouter>);
    });
    return { eventListInstance, component };
}

async function RenderistLite() {
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component;
    let eventListInstance;
    let schedulingUnit = {  task_blueprints_ids:[ 1,2,3,4]}
    await act(async () => {
        component = render(<MemoryRouter><SystemEventListLite history={{}} schedulingUnit={schedulingUnit}  location="/systemevent/list/" ref={(instance) => { eventListInstance = instance; } } /></MemoryRouter>);
    });
    return { eventListInstance, component };
}


