

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, fireEvent } from '@testing-library/react';
import { remove404Errors } from "../utils/test.helper";

import systemeventview from '../__mocks__/system_event.view.json'
import systemeventviewtemplate from '../__mocks__/system_event.view.template.json'
import taskblueprint from '../__mocks__/task.blueprint.json'


import { MockAxiosSecurity  } from './axiosmocks.'
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { SystemEventView } from "../routes/SystemEvent/system.event.view";
import { MemoryRouter } from "react-router-dom/cjs/react-router-dom.min";
import { giveConsistentNodes } from "../utils/test.helper";


function MakePrimaryMock() {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    let mock = new MockAdapter(axios);
    mock.onGet("/api/system_event/5").reply(200, systemeventview);
    mock.onGet("/api/system_event_template/5").reply(200, systemeventviewtemplate);
    mock.onGet("api/task_blueprint/?expand=scheduling_unit_blueprint,subtasks&fields=id,url,name,short_description,on_sky_start_time,on_sky_stop_time,scheduling_unit_blueprint,scheduling_unit_blueprint.id,scheduling_unit_blueprint.name,subtasks.id,subtasks.primary&id=1076&limit=1&offset=0").reply(200, taskblueprint);
    return mock

}

describe('Systen Event View ', () => {

       

    it('Systen Event View   renders correctly without security', async () => {
        let mock= new MockAdapter(axios);
        remove404Errors();
        mock.onGet("/").reply(404);
        let {  component } = await RenderView(false);
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('Systen Event View  correctly with data but no security to view and list', async () => {
        remove404Errors();
        let {  component } = await RenderView(true);
        expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot();
        
        component.unmount();
    });

    it('Systen Event View  correctly with data and security', async () => {
        let {  component } = await RenderView(true,true);
        expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot();
        component.unmount();
    });

    it('Systen Event View  correctly with data and security and stoptime', async () => {
        
        let {  component } = await RenderView(true,true,true);
        expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot();
        component.unmount();
    });

    it('Systen Event View  Closes the System Event', async () => {
        
        let {  component } = await RenderView(true,true);
        let closeBUtton = component.queryByTitle("Click to Close System Event");
        await act(async () => { fireEvent.click(closeBUtton); });
        expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot();
        component.unmount();
    });





    
});

async function RenderView(isWisecurity=false,iswithMock=false, isAlterNateBluePrint=false) {
    
    if (!isWisecurity && iswithMock) {
         MakePrimaryMock();
    }
    if (isWisecurity && iswithMock) {
        let mock = MockAxiosSecurity(MakePrimaryMock());

        if (isAlterNateBluePrint) {
            const systemeventviewWithStop = { ...systemeventview}
            systemeventviewWithStop.stop="2022-03-17T08:45:00"
            mock.onGet("/api/system_event/5").reply(200, systemeventviewWithStop);
        }
        await PermissionStackUtil.getPermissions(true);
    }
    
    let component;
    let Instance;
    let matchparams ={params: {id:5}}
    await act(async () => {
        component = render(<MemoryRouter><SystemEventView history={{}} match={matchparams}  location="/systemevent/view/5" ref={(instance) => { Instance = instance; } } /> </MemoryRouter>);
    });
    return { Instance, component };
}


