

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act,waitFor,fireEvent } from '@testing-library/react';
import  {TaskEditSecure}     from "../routes/Task/edit.secure";

import tasktemplates from '../__mocks__/task.templates.json'
import taskdraft from '../__mocks__/task.draft.one.json'
import refresolved from '../__mocks__/task.template.beamforming.refresolved.json'
import schedulingunitdraft from '../__mocks__/scheduling_unit_draft.json'
import validate_specification_doc from '../__mocks__/task.template.validate_specification_doc.json'
import { fakeauthanswer } from './axiosmocks.'
import { MemoryRouter } from "react-router-dom";
import { TaskEdit } from "../routes/Task";


function MakePrimaryMock(isAllowed) {
    let mock = new MockAdapter(axios);
    mock.onPut("http://localhost:3000/api/task_template/8/validate_specifications_doc").reply(200, validate_specification_doc);
    mock.onGet("/api/task_template/").reply(200, tasktemplates);
    mock.onGet("/api/task_draft/1").reply(200, taskdraft);
    mock.onGet("/api/task_draft/1/predecessors/").reply(200, []);
    mock.onGet("/api/task_draft/1/successors/").reply(200, []);
    mock.onGet("/api/task_template/8/ref_resolved_schema").reply(200, refresolved);
    mock.onGet("/api/scheduling_unit_draft/1/?expand=task_drafts,task_drafts.specifications_template").reply(200, schedulingunitdraft);
    if (isAllowed) {
        mock.onGet("/api/task_draft/1/?fields=name&limit=1").reply(200, fakeauthanswer, { "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, copy, create_task_blueprint, create_task_blueprint_and_subtasks, predecessors, successors" });
    }
    else {
        mock.onGet("/api/task_draft/1/?fields=name&limit=1").reply(200, fakeauthanswer, { "access-control-allow-methods": "" });
    }


    mock.onGet("/api/task_draft/1/task_blueprint/?fields=id,name").reply(200, {
        data: {
            "count": 1,
            "next": null,
            "previous": null,
            "results": [
                {
                    "id": 1,
                    "name": "Observation"
                }
            ]
        }
    });


    return mock

}

async function getTaskEditPage() {
    let component
    await act(async () => { component = render(<MemoryRouter><TaskEdit match={{ params: { id: "1" } }} history={{}} location="/task/edit/draft/1" /></MemoryRouter>) });

    const componentSnapshot = component.asFragment();
    component.unmount();
    return componentSnapshot
}
describe('Task Edit', () => {

    it('Task Edit  renders correctly without proper security', async () => {
        MakePrimaryMock(false);
        let renderer = getTaskEditPage();
        expect(renderer).toMatchSnapshot();
    });

    it('Task Edit  renders correctly with data', async () => {
        MakePrimaryMock(true);
        let security ={
            "1":{
            cancel:false,
            delete:true,
            edit:true,
            obsolete:false,
            view:true
            }
        }
        let component
        await act(async () => { component = render(<MemoryRouter><TaskEditSecure taskId="1" userrole={security}  history={{}} location="/task/edit/draft/1" /></MemoryRouter>) });
        
    
        
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });
});



it.skip("test  Task Draft edit", async () => {
    MakePrimaryMock();
    let content;
    await act(async () => {
        content = render(<MemoryRouter><TaskEdit match={{params:{id: "1"}}} history={{}} location="/task/edit/draft/1" /></MemoryRouter>);
        await new Promise((r) => setTimeout(r, 1000));
    });
         await waitFor(() => {
       //     const element = component.container.querySelector('.je-object__container');
       //     expect(element).toBeInTheDocument();
          },{timeout: 15000});
       

    // premery check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    const taskDescInputEl = content.queryByTestId('taskDescription');
    const taskDescInputEllValue = taskDescInputEl.value;
    await act( async() => {
        fireEvent.change(taskDescInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(taskDescInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(taskDescInputEl, { target: { value: taskDescInputEllValue } });
        await new Promise((r) => setTimeout(r, 500));
    });

    const angle1El = content.getByLabelText("specification[calibrator][pointing][angle1]");
    expect(angle1El.value).not.toBe('');
    const averageTSValue = angle1El.value;
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '82h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle1El, { target: { value: averageTSValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
 
    const angle2El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]");
    expect(angle2El.value).not.toBe('');
    const angle2ElValue = angle2El.value;
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '91d15m50.8s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle2El, { target: { value: angle2ElValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
 
    //Save task
     await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    
});